# -*- coding: utf-8 -*-
# @Time : 2023/7/4
# @Author : chengwenping2
# @File : test_case.py
"""
文件说明：添加测试用例，输出测试报告
"""
import html
import os
import sys
import copy
import json
import pyjson5
import traceback
from datetime import datetime

from pig_frame import log
from pig_frame.asserts import equal, not_equal, contain, not_contain
from pig_frame.common import get_project_home, jmespath
from pig_frame.http_request import request
from pig_frame.template import get_report_template
from pig_frame.num_util import calc_rate
from pig_frame.exceptions import PigFrameCaseStopException

traceback = traceback


def read_log(file):
    """
    读取执行日志
    :return: 日志内容
    :param file 调用方脚本名称
    """
    try:
        project_home = get_project_home()
        file_name = os.path.basename(file).split(".")[0]
        log_file = f"{project_home}{os.sep}logs{os.sep}{file_name}.log"
        if not os.path.exists(log_file):
            # 标准工作目录，读取 global.json里面的配置，主要用于服务器部署
            with open(f"{project_home}/conf/global.json", "r", encoding="utf-8") as f:
                global_conf = pyjson5.load(f)
                project_home = global_conf["log_path"]
        log_file = f"{project_home}{os.sep}logs{os.sep}{file_name}.log"

        with open(log_file, "r", encoding="utf-8") as f:
            return f.read()
    except:
        log.warn("获取执行日志发生异常，测试报告中将无法查看执行日志！")
        return "未获取到执行过程中的日志文件"

def check_report_name(report_name):
    try:
        if report_name is None or report_name == "":
            return "测试报告"
        else:
            for item in '<>/\|:"*?:：《》"*？ ,、、，,！@#￥%……&*（）!@#$%^&*()~`~`':
                report_name = str(report_name).replace(item, "")
            return report_name
    except:
        return "测试报告"

def check_python_ver():
    ver = sys.version_info
    if ver.major < 3:
        log.error("You should Upgrade to Python3.9+")
        exit(1)
    if ver.major == 3 and ver.minor < 9:
        log.error("You should Upgrade to Python3.9+")
        exit(1)

class TestCase:
    def __init__(self, report_name, creator="system", output_json_file=True):
        check_python_ver()
        # 测试报告名称
        self.report_name = check_report_name(report_name)
        # 验证点集合
        self.report = []
        # 测试报告路径
        self.report_path = ""
        self.pass_case = 0
        self.fail_case = 0
        self.start_time = datetime.now()
        # 脚本编写负责人
        self.creator = creator
        self.created = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        self.output_json_file = output_json_file
        self.end_time = None
        self.stop_reason = None

    def stop(self, stop_reason: str):
        self.stop_reason = stop_reason
        raise PigFrameCaseStopException(stop_reason)

    def check(self, response=None, **kwargs):
        """
        添加测试用例验证点
        :param response:调用http_request.request请求返回的数据对象
        :param kwargs:
        case_name:测试用例名
        check:期望值
        case_assert:断言结果对象、支持传入多个如[("case_name",check1)、("case_name",check2)] or equal() or [equal(),not_equal()]
        :return:
        """
        try:
            if response is None:
                response = {
                    "url": "",
                    "params": "",
                    "response": "",
                    "use_time": "",
                    "description": "",
                    "headers": {},
                    "status_code": "",
                    "method": "",
                    "response_time": "",
                    "retry_times": "",
                }
            if response:
                if isinstance(kwargs.get("case_assert"), list):
                    for sub_case_assert in kwargs.get("case_assert"):
                        if isinstance(sub_case_assert, tuple):
                            if len(sub_case_assert) == 2:
                                case_name = sub_case_assert[0]
                                case_check = sub_case_assert[1]
                            else:
                                case_name = ""
                                case_check = {
                                    "check_result": False,
                                    "check_detail": ["未填写断言方式"],
                                    "check_type": "contain",
                                    "expect": "",
                                    "actual": "",
                                    "filter_keys": [],
                                    "limit_keys": [],
                                }
                        else:
                            if len(kwargs.get("case_assert")) > 1:
                                case_name = (
                                    kwargs.get("case_name")
                                    + f"_{sub_case_assert.get('check_type')}"
                                )
                            else:
                                case_name = kwargs.get("case_name")
                            case_check = sub_case_assert
                        temp_check = {
                            "name": case_name,
                            "expect": case_check.get("expect"),
                            "actual": case_check.get("actual"),
                            "check_result": case_check.get("check_result"),
                            "check_type": case_check.get("check_type"),
                            "check_detail": case_check.get("check_detail"),
                            "filter_keys": case_check.get("filter_keys"),
                            "limit_keys": case_check.get("limit_keys"),
                            "url": response.get("url"),
                            "params": response.get("params"),
                            "response": response.get("response"),
                            "use_time": response.get("use_time"),
                            "description": response.get("description"),
                            "headers": eval(str(response.get("headers"))),
                            "status_code": response.get("status_code"),
                            "method": response.get("method"),
                            "response_time": response.get("response_time"),
                            "retry_times": response.get("retry_times"),
                        }
                        self.report.append(copy.deepcopy(temp_check))
                        log.debug(f"{case_name} 验证结果：")
                        log.debug(f'  断言结果:{case_check.get("check_result")}')

                        if not case_check.get("check_result"):
                            log.debug(f'  差异明细:{case_check.get("check_detail")}')
                else:
                    result = {
                        "name": kwargs.get("case_name"),
                        "expect": kwargs.get("case_assert").get("expect"),
                        "actual": kwargs.get("case_assert").get("actual"),
                        "check_result": kwargs.get("case_assert").get("check_result"),
                        "check_type": kwargs.get("case_assert").get("check_type"),
                        "check_detail": kwargs.get("case_assert").get("check_detail"),
                        "filter_keys": kwargs.get("case_assert").get("filter_keys"),
                        "limit_keys": kwargs.get("case_assert").get("limit_keys"),
                        "url": response.get("url"),
                        "params": response.get("params"),
                        "response": response.get("response"),
                        "use_time": response.get("use_time"),
                        "description": response.get("description"),
                        "headers": eval(str(response.get("headers"))),
                        "status_code": response.get("status_code"),
                        "method": response.get("method"),
                        "response_time": response.get("response_time"),
                        "retry_times": response.get("retry_times"),
                    }
                    self.report.append(copy.deepcopy(result))
                    log.debug(f'{kwargs.get("case_name")} 验证结果：')
                    log.debug(f'  断言结果:{kwargs.get("case_assert").get("check_result")}')

                    if not kwargs.get("case_assert").get("check_result"):
                        log.debug(
                            f'  差异明细:{kwargs.get("case_assert").get("check_detail")}'
                        )

            else:
                if isinstance(kwargs.get("case_assert"), list):
                    for sub_case_assert in kwargs.get("case_assert"):
                        if isinstance(sub_case_assert, tuple):
                            if len(sub_case_assert) == 2:
                                case_name = sub_case_assert[0]
                                case_check = sub_case_assert[1]
                            else:
                                case_name = ""
                                case_check = {
                                    "check_result": False,
                                    "check_detail": ["未填写断言方式"],
                                    "check_type": "contain",
                                    "expect": "",
                                    "actual": "",
                                    "filter_keys": [],
                                    "limit_keys": [],
                                }
                        else:
                            if len(kwargs.get("case_assert")) > 1:
                                case_name = (
                                    kwargs.get("case_name")
                                    + f"_{sub_case_assert.get('check_type')}"
                                )
                            else:
                                case_name = kwargs.get("case_name")
                            case_check = sub_case_assert
                        temp_check = {
                            "name": case_name,
                            "expect": case_check.get("expect"),
                            "actual": case_check.get("actual"),
                            "check_result": case_check.get("check_result"),
                            "check_type": case_check.get("check_type"),
                            "check_detail": case_check.get("check_detail"),
                            "filter_keys": case_check.get("filter_keys"),
                            "limit_keys": case_check.get("limit_keys"),
                            "url": response.get("url"),
                            "params": response.get("params"),
                            "response": response.get("response"),
                            "use_time": response.get("use_time"),
                            "description": response.get("description"),
                            "headers": "",
                            "status_code": "",
                            "method": "",
                            "response_time": response.get("response_time"),
                            "retry_times": response.get("retry_times"),
                        }
                        self.report.append(copy.deepcopy(temp_check))
                        log.debug(f"{case_name} 验证结果：")
                        log.debug(f'  断言结果:{case_check.get("check_result")}')

                        if not case_check.get("check_result"):
                            log.debug(f'  差异明细:{case_check.get("check_detail")}')
                    else:
                        result = {
                            "name": kwargs.get("case_name"),
                            "expect": kwargs.get("case_assert").get("expect"),
                            "actual": kwargs.get("case_assert").get("actual"),
                            "check_type": kwargs.get("case_assert").get("check_type"),
                            "check_result": kwargs.get("case_assert").get(
                                "check_result"
                            ),
                            "check_detail": kwargs.get("case_assert").get(
                                "check_detail"
                            ),
                            "filter_keys": kwargs.get("case_assert").get("filter_keys"),
                            "limit_keys": kwargs.get("case_assert").get("limit_keys"),
                            "url": response.get("url"),
                            "params": response.get("params"),
                            "response": response.get("response"),
                            "use_time": response.get("use_time"),
                            "description": response.get("description"),
                            "headers": "",
                            "status_code": "",
                            "method": "",
                            "response_time": response.get("response_time"),
                            "retry_times": response.get("retry_times"),
                        }
                        self.report.append(result)

                        log.debug(f'{kwargs.get("case_name")} 验证结果：')
                        log.debug(
                            f'  断言结果:{kwargs.get("case_assert").get("check_result")}'
                        )
                        if not kwargs.get("case_assert").get("check_result"):
                            log.debug(
                                f'  差异明细:{kwargs.get("case_assert").get("check_detail")}'
                            )
        except Exception as e:
            log.error(f"添加测试用例失败:{traceback.format_exc()}\n{e}")

    def run_ddt(self, host, kwargs):
        """
        执行外置Excel 测试用例
        @param host:
        @param kwargs:
        @return:
        """
        headers = kwargs.get("headers")
        excel_case = kwargs.get("pig_frame_ddt_http_case_list")
        if headers is None:
            headers = {}
        for case in excel_case:
            result = request(
                method=case.get("method"),
                url=f'{host}{case.get("path")}',
                json=case.get("body"),
                params=case.get("params"),
                data=case.get("data"),
                headers=headers,
                description=case.get("desc"),
            )
            if case.get("assert_type") == "相等":
                self.check(
                    result,
                    case_name=case.get("case_name"),
                    case_assert=equal(
                        case.get("exp"), jmespath.search("response", result)
                    ),
                )
            elif case.get("assert_type") == "不‍相等":
                self.check(
                    result,
                    case_name=case.get("case_name"),
                    case_assert=not_equal(
                        case.get("exp"), jmespath.search("response", result)
                    ),
                )
            elif case.get("assert_type") == "包含":
                self.check(
                    result,
                    case_name=case.get("case_name"),
                    case_assert=contain(
                        case.get("exp"), jmespath.search("response", result)
                    ),
                )
            elif case.get("assert_type") == "不包含":
                self.check(
                    result,
                    case_name=case.get("case_name"),
                    case_assert=not_contain(
                        case.get("exp"), jmespath.search("response", result)
                    ),
                )

    def check_without_request(
        self, description, case_name, case_assert: dict, use_time
    ):
        """
        添加测试用例验证点，用于无请求的验证点
        :param description 请求名称
        :param case_name 验证点名称
        :param case_assert 验证结果
        :param use_time 花费时间
        """
        try:
            response = {
                "url": "",
                "params": "",
                "response": "",
                "use_time": use_time,
                "description": description,
                "headers": {},
                "status_code": "",
                "method": "",
                "response_time": "",
                "retry_times": "",
            }
            result = {
                "name": case_name,
                "expect": case_assert.get("expect"),
                "actual": case_assert.get("actual"),
                "check_type": case_assert.get("check_type"),
                "check_result": case_assert.get("check_result"),
                "check_detail": case_assert.get("check_detail"),
                "filter_keys": case_assert.get("filter_keys"),
                "limit_keys": case_assert.get("limit_keys"),
                "url": response.get("url"),
                "params": response.get("params"),
                "response": response.get("response"),
                "use_time": response.get("use_time"),
                "description": response.get("description"),
                "headers": "",
                "status_code": "200",
                "method": "",
                "response_time": response.get("response_time"),
                "retry_times": response.get("retry_times"),
            }
            self.report.append(result)

            log.debug(f"{case_name} 验证结果：")
            log.debug(f'  断言结果:{case_assert.get("check_result")}')
        except Exception as e:
            log.error(f"添加测试用例失败:{traceback.format_exc()}\n{e}")

    def output_report(self, file=None, unique=True, suite=None, task=None):
        """
        基线数据比对、测试报告生成、测试报告数据写入DB
        :return:
        :param file 调用方脚本名称
        :param unique 多次执行是否只生成一份报告
        """
        project_home = get_project_home()

        try:
            pass_case = 0
            fail_case = 0
            create_time = self.created
            if file is not None:
                report_home = os.path.dirname(file).replace("testcases", "test_report")
            else:
                report_home = f"{project_home}{os.sep}test_report"

            os.makedirs(report_home, exist_ok=True)
            if unique is True:
                self.report_path = f"{report_home}{os.sep}{self.report_name}.html"
            else:
                self.report_path = f'{report_home}{os.sep}{self.report_name}_{datetime.now().strftime("%Y-%m-%d_%H%M%S_%f")[:-3]}.html'
            for i in range(len(self.report)):
                self.report[i]["id"] = i
                if self.report[i]["check_result"]:
                    pass_case += 1
                    self.report[i]["check_result"] = "通过"
                else:
                    fail_case += 1
                    self.report[i]["check_result"] = "失败"
            use_time = datetime.now() - self.start_time
            self.end_time = datetime.now()

            logs = ""
            if file is not None:
                logs = html.escape(read_log(file))
            if logs == "未获取到执行过程中的日志文件":
                if suite is not None:
                    logs = html.escape(read_log(suite))
            if logs == "未获取到执行过程中的日志文件":
                if task is not None:
                    logs = html.escape(read_log(task))

            # 生成报告前，转义报告数据
            for index, value in enumerate(self.report):
                if isinstance(value["response"], str):
                    self.report[index]["response"] = html.escape(value['response'])

                    self.report[index]["check_result"] = html.escape(value['check_result'])
                    self.report[index]["check_detail"] = [html.escape(i) for i in value['check_detail']] if isinstance(value['check_detail'], list) else value['check_detail']

                    self.report[index]["actual"] = html.escape(value['actual']) if isinstance(value['actual'], str) else value['actual']
                    self.report[index]["expect"] = html.escape(value['expect']) if isinstance(value['expect'], str) else value['expect']
            allData = {
                "createTime": create_time,
                "tableData": self.report,
                "title": self.report_name,
                "totalTime": round(use_time.total_seconds(), 2),
                "pass": pass_case,
                "fail": fail_case,
                "passrate": calc_rate(pass_case, fail_case),
                "total": fail_case + pass_case,
                "logs": logs,
                "stop_reason": self.stop_reason
            }
            report_lines = get_report_template(
                case_detail=json.dumps(
                    obj=allData,
                    default=lambda x: x.__dict__,
                    sort_keys=False,
                    indent=2,
                    ensure_ascii=False,
                ),
                case_pass=pass_case,
                case_fail=fail_case
            )
            with open(self.report_path, "w", encoding="utf-8") as f:
                f.write(report_lines)
            if self.output_json_file:
                with open(
                    self.report_path.replace(".html", ".json"), "w", encoding="utf-8"
                ) as f:
                    f.write(
                        json.dumps(
                            obj=allData,
                            default=lambda x: x.__dict__,
                            sort_keys=False,
                            indent=2,
                            ensure_ascii=False,
                        )
                    )
            self.pass_case = pass_case
            self.fail_case = fail_case
            log.info(f"{self.report_name} 执行结果：")
            log.info(f"  用例共计: {self.pass_case+self.fail_case} 条")
            log.info(f"  通过: {self.pass_case} 条")
            if self.fail_case > 0:
                log.error(f"  失败: {self.fail_case} 条")
            log.info(f"  测试报告已生成至：{self.report_path}")
            return self.report_path
        except Exception as e:
            log.error(f"生成报告错误：%s{str(e)}", traceback.format_exc())


if __name__ == "__main__":
    print(check_report_name("测试/、、\\yosh"))



# -*- coding: utf-8 -*-
# @Time : 2023/7/7
# @Author : chengwenping2

from pig_frame.asserts import *
from pig_frame.decorators import pig_frame_case
from pig_frame.common import *
from pig_frame.http_request import request
from pig_frame.test_case import TestCase


@pig_frame_case(report_name="接口返回值是否正确", creator="负责人")
def case_template(test_case: TestCase, **kwargs):

    result = request(
        method="get",
        url="https://postman-echo.com/get",
        description="Postman Echo",
        json=None,
        data=None,
        params={"foo1": "bar1", "foo2": "bar2"},
        headers=None,
    )

    test_case.check(
        result,
        case_name="入参原样返回",
        case_assert=equal(
            {"foo1": "bar1", "foo2": "bar2"}, jmespath.search("response.args", result)
        ),
    )
    test_case.stop("用户主动结束用例")

    test_case.check(
        result,
        case_name="入参bar1原样返回",
        case_assert=equal("bar1", jmespath.search("response.args.foo1", result)),
    )


if __name__ == "__main__":
    kwargs = {}
    # env_config_file:环境配置文件相对路径（相对于conf/），例如/conf/uat.json就写uat.json，/conf/product/test.json就写product/test.json
    case_template(env_config_file="uat.json", **kwargs)



# -*- coding: utf-8 -*-
# @Time : 2023/7/7
# @Author : chengwenping2


from pig_frame.http_request import *
from pig_frame.asserts import *
from pig_frame.decorators import pig_frame_case


@pig_frame_case(report_name="test_demo", creator="程文平")
def tset(test_case, **kwargs):
    bar2 = kwargs.get("bar2")
    bar1 = kwargs.get("bar1")
    result = request(
        method="get",
        url="https://postman-echo.com/get",
        description="Postman Echo",
        json=None,
        data=None,
        params={"foo1": bar1, "foo2": bar2},
        headers=None,
    )

    test_case.check(
        result,
        case_name="Http 返回码为 200",
        case_assert=[
            (
                "返回码200",
                equal(
                    200,
                    jmespath.search("status_code", result),
                    filter_keys=None,
                    limit_keys=None,
                ),
            ),
            (
                "返回码不是200",
                not_equal(
                    200,
                    jmespath.search("status_code", result),
                    filter_keys=None,
                    limit_keys=None,
                ),
            ),
        ],
    )

    test_case.check(
        result,
        case_name="Http 返回码为 200",
        case_assert=[
            equal(
                200,
                jmespath.search("status_code", result),
                filter_keys=None,
                limit_keys=None,
            ),
            not_equal(
                200,
                jmespath.search("status_code", result),
                filter_keys=None,
                limit_keys=None,
            ),
        ],
    )
    test_case.check(
        result,
        case_name="Http 返回码为 200",
        case_assert=equal(
            200,
            jmespath.search("status_code", result),
            filter_keys=None,
            limit_keys=None,
        ),
    )


if __name__ == "__main__":
    kwargs = {}
    # env_config_file:环境配置文件相对路径（相对于conf/），例如/conf/uat.json就写uat.json，/conf/product/test.json就写product/test.json
    tset(env_config_file="test.json", **kwargs)



# -*- coding: utf-8 -*-
# @Time : 2023/7/4
# @Author : chengwenping2
import io
import json
import time

import json5

from pig_frame.common import get_project_home


def tail_f(file_name, callback, interval=0.1):
    """
    Unix tail follow implementation in Python.can be used to monitor changes to a file.
    :param file_name: 文件名称
    :param callback: If a callback function is registered it is called with every new line
    :param interval: Number of seconds to wait between each iteration; Defaults to 0.1.
    """
    with open(file_name) as file_:
        # Go to the end of file
        file_.seek(0, io.SEEK_END)
        while True:
            curr_position = file_.tell()
            line = file_.readline()
            if not line:
                file_.seek(curr_position)
            else:
                callback(line)
            time.sleep(interval)


def tail(file_name, line_count=10, encoding="utf-8"):
    """
    读取某文本文件最后 N 行
    :param file_name: 文件名称
    :param line_count: 读多少行
    :param encoding: 文件编码
    :return: 数组格式的行列表
    """
    f = open(file_name, mode="rb")
    f.seek(0, io.SEEK_END)
    file_size = f.tell()
    if file_size == 0 or line_count <= 0:
        return []

    lines = []
    prev_char = None
    curr_line = bytearray()
    chars_read = 0
    f.seek(-1, io.SEEK_END)
    while True:
        curr_char = f.read(1)
        chars_read += 1
        # 以下三个步骤：增加字符、增加行、跳出循环，如果文件已经读完，则都要做
        if curr_char not in (b"\n", b"\r") or chars_read == file_size:
            curr_line.extend(curr_char)
        if (
            curr_char == b"\n"
            or (curr_char == b"\r" and not prev_char == b"\n")
            or chars_read == file_size
        ):
            curr_line.reverse()
            lines.append(bytes(curr_line).decode(encoding))
            curr_line.clear()
        if len(lines) == line_count or chars_read == file_size:
            break
        # 往前退一个字节
        f.seek(-2, io.SEEK_CUR)
        prev_char = curr_char
    lines.reverse()

    f.close()
    return lines


def save_config_by_json(file_name, key, value):
    """
    保存配置到Json格式的配置文件，有就更新，没有就新增

    :param file_name:文件名称，环境配置文件相对路径（相对于conf/），例如/conf/pre.json就写pre.json，/conf/product/test.json就写product/test.json
    :param key: 配置项名称
    :param value: 配置值
    :return True/False
    """
    return save_configs_by_json(file_name,  key, value)


def save_configs_by_json(file_name, key, value):
    """
    保存配置到Json格式的配置文件，有就更新，没有就新增
    :param file_name:文件名称，环境配置文件相对路径（相对于conf/），例如/conf/pre.json就写pre.json，/conf/product/test.json就写product/test.json
    :param key_values: key：value 格式的字典
    :return True/False
    """
    try:
        file_name = get_project_home() + "/conf/" + file_name
        f = open(file_name, "r", encoding="utf-8")
        old = json5.load(f)
        f.close()

        list = key.split(".")
        old[list[0]][list[1]] = value

        f = open(file_name, "w", encoding="utf-8")
        # rs = {**old, **key_values}
        f.write(json.dumps({**old}, indent=2, ensure_ascii=False))
        f.close()
        return True
    except:
        return False




# -*- coding: utf-8 -*-
# @Time : 2023/07/04
# @Author : chengwenping2

class PigFrameCaseException(Exception):
    def __init__(self, description, case_name, check_detail, use_time):
        self.description = description
        self.case_name = case_name
        self.check_detail = check_detail
        self.use_time = use_time
        super(PigFrameCaseException, self)


class PigFrameCaseStopException(Exception):
    def __init__(self, stop_reason: str):
        self.stop_reason = stop_reason
        super(PigFrameCaseStopException, self)



# -*- coding: utf-8 -*-
# @Time : 2023/07/04
# @Author : chengwenping2
# @File : common.py
"""
文件说明：封装常用公共方法
"""
import copy
import datetime
import hashlib
import json
import os
import platform
import random
import re
import socket
import string as string_char
import traceback
import uuid
import getpass

import psutil
from faker import Faker

import jmespath
import jsonpath
import pyjson5 as json5

from pig_frame import log
from pig_frame.template import get_mail_html
from pig_frame.xmind_util import XmindUtil
import sys

jsonpath = jsonpath
random = random
traceback = traceback
logging = log
jmespath = jmespath
faker = Faker(locale="zh_CN")


def get_project_home():
    """
    获取符合 pig_frame 标准框架项目的项目根目录
    :return: str
    """
    # 进程当前运行目录
    cwd = os.getcwd()
    index = cwd.rfind(f"{os.sep}testcases")
    if index == -1:
        return cwd

    project_home = cwd[:index]
    return project_home


def get_case_home():
    """
    获取项目用例根目录
    @return:
    """
    return f"{get_project_home()}{os.sep}/testcases"


def add_traceback(param):
    pass


def get_conf_home():
    """
    获取配置根目录
    @return:
    """
    return f"{get_project_home()}{os.sep}/conf"


def process_str_pystr(string):
    """
    字符串变量处理
    :param string:
    :return:
    """
    try:
        if isinstance(string, str):
            match = re.findall("\$\{\{.*\}\}", string)
            if len(match) != 0:
                for match_item in match:
                    string = string.replace(match_item, match_item[3:-2])
                return string
            match = re.findall("\$\{.*\}", string)
            if len(match) != 0:
                string = string.replace("${", "{")
                return f'f"{string}"'
            return f'"{string}"'
        else:
            return string
    except:
        return string


def process_obj_variable(obj, result=None):
    keys = dict_to_simple(obj)
    replace_value = {}
    for key in keys:
        try:
            value = jmespath.search(key, obj)
            if not isinstance(value, str):
                continue
            match = re.findall("\${{\w+}}", value)
            if len(match) != 0:
                replace_value[f"'{value}'"] = process_str_pystr(value)
            else:
                match = re.findall("\${\w+}", value)
                if len(match) != 0:
                    replace_value[f"'{value}'"] = process_str_pystr(value)
        except:
            continue
    obj_str = str(obj)
    for key in replace_value:
        obj_str = obj_str.replace(key, replace_value[key])
    return obj_str


def process_variable(obj):
    """
    带参数的json字符串转换为python对象字符串
    :param string:json字符串
    :return:str
    """
    try:
        if isinstance(obj, str):
            return process_str_pystr(obj)
        elif isinstance(obj, dict) or isinstance(obj, list):
            return process_obj_variable(obj)
        else:
            return obj
    except:
        return obj
#
# def merge_dict(test_case, pig_frame_env):
#     """
#     老版本脚本兼容
#     """
#     return merge_dict_with_self_variables(test_case, pig_frame_env, **{})


def _exclude(file: str, exclude: list[str]):
    if exclude is None or len(exclude) == 0:
        return False
    for item in exclude:
        if item in file:
            return True
    return False


def get_cases_by_folder(folder_path, exclude: list[str] = None, result=None):
    """
    查找所有测试用例
    @param folder_path:
    @param result:
    @param exclude:需要排除的文件
    @return:
    """
    if result is None:
        result = []
    files = os.listdir(folder_path)
    for file in files:
        try:
            if _exclude(file, exclude):
                log.debug(f"{file} 已被过滤.")
                continue
            new_path = folder_path + os.sep + file
            if os.path.isdir(new_path):
                get_cases_by_folder(new_path, exclude, result)
            else:
                if file.endswith(".py") or file.endswith(".PY"):
                    if new_path.__contains__("testcases"):
                        result.append(new_path)
        except:
            continue
    return result


def merge_dict_with_self_variables(test_case, pig_frame_env, **config):
    """
    变量优先级处理：用例变量>环境变量>全局变量
    :param test_case: 用例执行脚本__file__
    :param pig_frame_env: 环境变量文件，相对于conf/目录
    :return: 按照优先级合并后的变量[]
    """
    global_variables = {}
    test_case_variables = {}
    env_variables = {}
    project_home = get_project_home()
    if not os.path.exists(f"{project_home}{os.sep}conf"):
        log.error("配置目录不存在，跳过变量优先级处理动作")
        return config

    try:
        file_name = str(test_case).split(f"testcases{os.sep}")[1].split(".")[0]
        case_file = f"{os.sep}conf{os.sep}{file_name}.json"
        if not os.path.exists(project_home + case_file):
            log.warn("用例变量[%s]不存在", case_file)
        else:
            try:
                f = open(project_home + case_file, "r", encoding="utf-8")
                test_case_variables = json5.load(f)
                log.debug("用例变量：%s", test_case_variables)
            except json5.Json5EOF:
                test_case_variables = {}
                log.error("读取用例变量[%s]发生错误", case_file)
    except:
        case_file = ""

    env_file = f"{os.sep}conf{os.sep}{pig_frame_env}"
    if os.path.exists(project_home + env_file):
        try:
            env_variables = json5.load(
                open(project_home + env_file, "r", encoding="utf-8")
            )
            log.debug("环境变量:%s", env_variables)
        except json5.Json5EOF:
            env_variables = {}
            log.error("读取环境变量[%s]发生错误", env_file)
    else:
        log.error("环境变量[%s]不存在", env_file)

    global_file = f"{os.sep}conf{os.sep}global.json"
    if os.path.exists(project_home + global_file):
        try:
            global_variables = json5.load(
                open(project_home + global_file, "r", encoding="utf-8")
            )
            log.debug("系统变量:%s", global_variables)
        except json5.Json5EOF:
            global_variables = {}
            log.error("读取系统变量[%s]发生错误", global_file)
    else:
        log.error("系统变量[%s]不存在", global_file)

    log.debug(
        "变量优先级：外部传递变量%s>用例变量%s>环境变量%s>全局环境%s", config, case_file, env_file, global_file
    )
    variables = {**global_variables, **env_variables, **test_case_variables, **config}
    log.info("变量合并结果(最终生效内容)：%s", variables)
    return variables


def remove_log(test_case):
    """
    删除用例的执行日志
    :param test_case: 用例执行脚本
    """
    project_home = get_project_home()
    try:
        file_name = os.path.basename(test_case).split(".")[0] if os.path.basename(test_case).split(".")[0] else test_case
        log_file = f"{project_home}{os.sep}logs{os.sep}{file_name}.log"
        if os.path.exists(log_file):
            os.remove(log_file)
    except FileNotFoundError:
        log.error(traceback.format_exc())


def compatible_jsonpath(key):
    """
    兼容jsonpath 表达式
    :param key:
    :return:
    """
    if str(key).startswith("$"):
        key = str(key).replace("$.", "").replace("$", "")
        match = re.findall("\.\d+", key)
        for item in match:
            key = str(key).replace(item, f'."{item[1::]}"')
    return key


def obj_search(obj, key):
    key = compatible_jsonpath(key)
    return jmespath.search(key, obj)


def filter_key(keys, key):
    """
    判断key是否在keys中存在，支持data.*、data[*]特殊可以值判断
    @param keys: jsonpath 查询语句写法key_list ['data.data.conde']
    @param key: key 字符串 data.data
    @return:
    """
    for i in range(len(keys)):
        keys[i] = compatible_jsonpath(keys[i])
    for item in keys:
        if str(key).startswith(f"{item}."):
            return True
        elif str(key).startswith(f"{item}["):
            return True
    if key in keys:
        return True
    else:
        for item in keys:
            if str(item).__contains__("[*]") and str(item).endswith(".*"):
                temp_index = [m.start() for m in re.finditer("\*", str(item)[0:-1])]
                for char_index in temp_index:
                    key = replace_char(key, "*", char_index)
                if str(key).startswith(str(item)[0:-1]):
                    return True
            elif str(item).endswith(".*"):
                if str(key).startswith(str(item)[0:-1]):
                    return True
            elif str(item).__contains__("[*]"):
                temp_index = [m.start() for m in re.finditer("\*", str(item))]
                for char_index in temp_index:
                    key = replace_char(key, "*", char_index)
                if str(item) == str(key):
                    return True
        return False


def limit_check_key(keys, key):
    """
    判断key是否在keys中存在，支持data.*、data[*]特殊可以值判断
    @param keys: jsonpath 查询语句写法key_list ['data.data.conde']
    @param key: key 字符串 data.data
    @return:
    """
    for i in range(len(keys)):
        keys[i] = compatible_jsonpath(keys[i])
    if len(keys) == 0:
        return True
    if key in keys:
        return True
    else:
        for item in keys:
            if str(item).__contains__("[*]") and str(item).endswith(".*"):
                temp_index = [m.start() for m in re.finditer("\*", str(item)[0:-1])]
                for char_index in temp_index:
                    key = replace_char(key, "*", char_index)
                if str(key).startswith(str(item)[0:-1]):
                    return True
            elif str(item).endswith(".*"):
                if str(key).startswith(str(item)[0:-1]):
                    return True
            elif str(item).__contains__("[*]"):
                temp_index = [m.start() for m in re.finditer("\*", str(item))]
                for char_index in temp_index:
                    key = replace_char(key, "*", char_index)
                if str(item) == str(key):
                    return True
            elif str(item).startswith("*"):
                return True
            elif (
                    str(key).startswith(item + ".")
                    or str(key).startswith(item + "[")
                    or str(key).startswith(f'"{item}"')
            ):
                return True
        return False


def replace_char(string, char, index):
    """
    替换字符串指定位置字符
    @param string:
    @param char:
    @param index:
    @return:
    """
    try:
        string = list(string)
        string[index] = char
        return "".join(string)
    except:
        return string


def check_str_ascii_letters(sting):
    for c in str(sting):
        if c not in string_char.ascii_letters:
            return True
    return False


def dict_to_simple(dict_data):
    if isinstance(dict_data, dict):
        return dict_to_simple_process(dict_data, temp="")
    else:
        return dict_to_simple_process(dict_data, temp="")


def dict_to_simple_process(dict_data, temp="", result=None):
    """
    将复杂对象扁平化处理，返回jsonpath查询语句
    :data 待处理对象
    :temp 中间临时变量，不需要传值
    :result 返回链式key [],调用方法时，避免数据污染，可以传入初始值[]
    """
    if result is None:
        result = []
    data = copy.deepcopy(dict_data)
    if isinstance(data, dict):
        for key in data:
            temp_key = key
            try:
                if not check_str_ascii_letters(temp_key):
                    dict_to_simple_process(data[key], f"{temp}{temp_key}.", result)
                else:
                    dict_to_simple_process(data[key], f'{temp}"{temp_key}".', result)
            except:
                dict_to_simple_process(data[key], f"{temp}{temp_key}.", result)

    if isinstance(data, list):
        for i in range(len(data)):
            if temp.endswith("."):
                temp = temp[0:-1]
            dict_to_simple_process(data[i], f"{temp}[{i}].", result)
    else:
        if isinstance(data, dict) or isinstance(data, list):
            pass
        else:
            if temp.endswith("."):
                temp = temp[0:-1]
            result.append(temp)
    return result


def case_to_xmind(report, report_name, creator, path, all_in_one=False):
    try:
        if all_in_one:
            pass
        else:
            api_details = []
            apis = []
            step_details = []
            step_detail = {}
            url = ""
            number = 0
            for step in report:
                try:
                    curl_url = step.get("url")
                    if curl_url == "":
                        continue
                    if (
                            url != curl_url
                            or f"{number}、" + step.get("description") not in step_detail
                    ):
                        step_details.append(copy.deepcopy(step_detail))
                        step_detail = {}
                        number += 1
                        step_detail[f"{number}、" + step.get("description")] = [
                            step.get("name")
                        ]
                    else:
                        step_detail[f"{number}、" + step.get("description")].append(
                            step.get("name")
                        )
                    url = curl_url
                    if curl_url not in apis:
                        api_details.append(
                            {
                                step.get("description"): [
                                    {"url": curl_url},
                                    {"params": step.get("params").get("params")},
                                    {"data": step.get("params").get("data")},
                                    {"json": step.get("params").get("json")},
                                ]
                            }
                        )
                        apis.append(curl_url)
                except:
                    continue
            xmind_detail = {
                report_name: [
                    {"测试步骤": step_details},
                    {"接口详情": api_details},
                    {"负责人": creator},
                ]
            }
            XmindUtil().create_xmind_file(
                data=xmind_detail, path=path, file_name=report_name
            )

            log.info(f"用例知识已沉淀至:{path}/{report_name}.xmind")
    except:
        log.info(f"用例知识已沉淀xmind文件生成失败")


def report_overview(results, upload=False):
    """
    批量执行测试用例时返回统计报告
    @param upload: 是否需要上传测试报告到oss
    @param results: 测试用例集执行结果
    @return:
    """
    detail, report_path = [], {}
    for report in results:
        try:
            if report is not None:
                # if upload:
                #     out_link = upload_report(report.report_path)
                #     report.report_path = out_link
                # else:
                #     out_link = report.report_path
                report_path[report.report_name] = report.report_path
                detail.append(
                    {
                        "case_name": report.report_name,
                        "report_path": report.report_path,
                        "creator": report.creator,
                        "pass_case": report.pass_case,
                        "fail_case": report.fail_case,
                        "use_time": round((report.end_time - report.start_time).total_seconds(), 2),
                        "start_time": report.start_time.strftime("%Y-%m-%d %H:%M:%S"),
                        "end_time": report.end_time.strftime("%Y-%m-%d %H:%M:%S"),
                        "result": report.report
                    }
                )
        except:
            log.info(f"生成汇总报告：{traceback.format_exc()}")
            continue

    return get_mail_html(
        **{
            "total_scene": len(detail),
            "fail_scene": len([item for item in detail if item["fail_case"] > 0]),
            "pass_case": sum(jmespath.search("[*].pass_case", detail)),
            "fail_case": sum(jmespath.search("[*].fail_case", detail)),
            "use_time": round(sum(jmespath.search("[*].use_time", detail)), 2),
            "detail": detail,
        }
    ), report_path


def create_overview_report(project_home, report_name, results, upload=False):
    """
    批量执行生成汇总测试报告
    @param report_name:
    @param results:
    @param upload:
    @return:
    """
    try:
        report_path = f"{project_home}{os.sep}test_report{os.sep}{report_name}.html"
        html = report_overview(results, upload)[0]
        with open(report_path, "w", encoding="utf-8") as file:
            file.write(html)
        results_new = []
        for item in results:
            if not isinstance(item, dict):
                results_new.append(
                    {
                        "created": item.created,
                        "creator": item.creator,
                        "pass_case": item.pass_case,
                        "report": item.report,
                        "report_name": item.report_name,
                        "report_path": item.report_path,
                    }
                )
        with open(report_path.replace(".html", ".json"), "w", encoding="utf-8") as file:
            file.write(
                json.dumps(
                    obj=results_new,
                    default=lambda x: x.__dict__,
                    sort_keys=False,
                    indent=2,
                    ensure_ascii=False,
                )
            )
        log.info(f"汇总测试报告已生成至:{report_path}")
        return f"{report_path}"
    except:
        log.info(f"生成汇总报告错误:{traceback.format_exc()}")
        return None


def process_file_name(file_name):
    """
    处理文件名称特殊字符，避免文件生成失败
    @param file_name:
    @return:
    """
    for item in '<>/\|:"*?:：《》"*？ ,、、，,！@#￥%……&*（）!@#$%^&*()~`~`':
        file_name = str(file_name).replace(item, "")
    return file_name


def check_case_path(case_path):
    """
    校验测试用例是否在testcases 目录下
    @param case_path:
    @return:
    """
    path = os.path.dirname(case_path)
    while True:
        if os.path.basename(path) == "testcases":
            if os.path.dirname(path) in sys.path:
                break
            sys.path.append(os.path.dirname(path))
            break
        elif os.path.basename(path) == "":
            print("请将用例放入testcases目录下执行:\n不在testcases目录下执行的用例、测试报告、日志等可能混乱")
            break
        else:
            path = os.path.dirname(path)

    print("")


def get_call_link(step=1):
    """
    获取脚本调用链路 并返回调用路径的指定上级
    @param step:相对于当前调用链路的前面几步
    @return:
    """
    try:
        path = os.path.abspath(__file__)
        traceback_extract_stack = traceback.extract_stack()
        for index, value in enumerate(traceback_extract_stack):
            if value.filename == path:
                return traceback_extract_stack[index - step - 1].filename
        return ""
    except:
        return ""


def get_log_conf(project_home):
    try:
        # 标准工作目录，读取 global.json里面的配置，用于服务器部署
        with open(f"{project_home}/conf/global.json", "r", encoding="utf-8") as f:
            global_conf = json5.load(f)
            return global_conf["log_path"]
    except:
        pass
    return project_home


def get_md5(string):
    """
    获取字符串的MD5 值
    @param string:
    @return:
    """
    m = hashlib.md5()
    m.update(string.encode("utf-8"))
    return m.hexdigest().upper()


def get_system_ip():
    """
    获取本机ip地址
    @return:
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(("8.8.8.8", 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip


def get_mac_address():
    """
    获取本机mac地址
    @return:
    """
    mac = uuid.UUID(int=uuid.getnode()).hex[-12:]
    return ":".join([mac[e: e + 2] for e in range(0, 11, 2)])


def get_system_info():
    """
    获取电脑主机关键信息
    @return:
    """
    platform_info = {
        "machine": platform.machine(),
        "node": platform.node(),
        "system": platform.system(),
        "platform": platform.platform(),
        "ip": get_system_ip(),
        "host_name": socket.gethostname(),
        "cpu_count": psutil.cpu_count(),
        "total_memory": f"{round(psutil.virtual_memory().total / 1024 / 1024 / 1024, 2)} G",
        "free_memory": f"{round(psutil.virtual_memory().free / 1024 / 1024 / 1024, 2)} G",
        "local_time": datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        "mac": get_mac_address(),
        "user": getpass.getuser()
    }
    return platform_info


def get_project_name():
    """
    获取项目名称，如果 conf/global.json中配置了"project_name"，则直接返回它，否则获取项目目录作为项目名称
    :return:
    """
    project_home = get_project_home()
    if os.path.exists(f"{project_home}/conf/global.json"):
        with open(f"{project_home}/conf/global.json", "r", encoding="utf-8") as f:
            global_conf = json5.load(f)
            return global_conf["project_name"]

    return os.path.basename(project_home)



# -*- coding: utf-8 -*-
# @Time : 2023/7/4
# @Author : chengwenping2
# @File : task.py
"""
文件说明：调度任务
"""

from apscheduler.schedulers.blocking import BlockingScheduler
from apscheduler.triggers.cron import CronTrigger


class Task:
    def __init__(self):
        self.scheduler = BlockingScheduler()

    def add_task(self, function, cron: str, args=()):
        """
        添加调度任务
        :param function: 需要执行的方法
        :param cron: cron 标准表达式
        :param args: 执行方法依赖参数
        :return:
        """
        self.scheduler.add_job(function, CronTrigger.from_crontab(cron), args=args)

    def start_tasks(self):
        self.scheduler.start()


if __name__ == '__main__':
    scheduler = BlockingScheduler()
    scheduler.add_job(lambda x:print(x), CronTrigger.from_crontab("13 0/1 * * *"), args=("1"))
    scheduler.start()



# -*- coding: utf-8 -*-
# @Time : 2023/7/7
# @Author : chengwenping2

from pig_frame.asserts import *
from pig_frame.decorators import pig_frame_case
from pig_frame.common import *
from pig_frame.http_request import request


@pig_frame_case(report_name="HTML请求DEMO", creator="负责人", unique_report=True)
def case_template(test_case, **kwargs):
    result = request(
        method="get",
        url="http://127.0.0.1:5001/html",
        description="本地Mock Server返回html"
    )

    log.info(f'html:{result["html_tree"]}')

    test_case.check(
        result,
        case_name="返回html验证",
        case_assert=equal('<meta charset="utf-8"/>', str(result["html_tree"].contents[1]))
    )
    test_case.check(
        result,
        case_name="返回的content-type为json",
        case_assert=contain("text/html", result["response_headers"]["Content-Type"])
    )


if __name__ == "__main__":
    kwargs = {}
    # env_config_file:环境配置文件相对路径（相对于conf/），例如/conf/uat.json就写uat.json，/conf/product/test.json就写product/test.json
    case_template(env_config_file="test.json", **kwargs)



# -*- coding: utf-8 -*-
# @Time : 2023/7/4
# @Author : chengwenping2
# @File : test_suite.py
"""
文件说明：可管理多个测试用例，批量执行测试用例
"""
import traceback
import os
from pig_frame import log
from pig_frame.runner import run_testcase_script
from pig_frame.my_thread import MyThread
from pig_frame.common import create_overview_report, get_cases_by_folder
from pig_frame.se_common import get_case_type

class TestSuite:
    def __init__(self, task_name="巡检集", suite_name="测试集", author="System", case_home=None):
        self.test_cases = []
        self.parallel = False
        self.suite_name = suite_name
        self.case_home = case_home
        self.author = author
        self.suite = f"{self.case_home}{os.sep}{self.suite_name}.py"
        self.task_name = task_name
        self.task = f"{self.case_home}{os.sep}{self.task_name}.py"

    def add_cases_by_folder(self, folder_path, env_config_file=None, exclude: list[str] = None, kwargs=None):
        """
        通过文件夹维度批量添加测试用例
        @param folder_path:用例目录绝对路径
        @param env_config_file:
        @param exclude: 排除条目，传递字符串数组
        @param kwargs:
        """
        if self.case_home is not None:
            folder_path = f"{self.case_home}{os.sep}{folder_path}"
        folder_path = os.path.abspath(folder_path)
        suite = f"{self.case_home}{os.sep}{self.suite_name}.py"
        cases = get_cases_by_folder(folder_path=folder_path, exclude=exclude)
        for item in cases:
            if get_case_type(item) in ["用例集", "未知"]:
                continue
            if kwargs is None:
                kwargs = {}
            self.add_case(item, env_config_file, **kwargs)

    def add_case(self, file, env_config_file=None, **kwargs):
        """
        添加测试用例
        :param file: 测试用例绝对路径
        :param env_config_file: 环境变量文件，相对于conf/目录
        :kwargs :用户自定义变量可通过此参数添加
        :return:
        """
        env_config_file = env_config_file or kwargs.pop('suite_env_config_file', 'test.json')
        if self.case_home is not None:
            if str(os.path.abspath(file)).startswith(os.path.abspath(self.case_home)):
                folder_path = file
            else:
                folder_path = f"{self.case_home}{os.sep}{file}"
        else:
            folder_path = file
        folder_path = os.path.abspath(folder_path)

        if str(env_config_file).__contains__("testcases"):
            env_config_file = str(env_config_file).split("testcases")[1]
        self.test_cases.append(
            {"case_file": folder_path, "case_args": env_config_file, "task":self.task, "suite":self.suite, "kwargs": kwargs}
        )

    def get_cases(self):
        return self.test_cases

    def run_cases(self):
        results = self._run_cases()
        return results

    def _run_cases(self):
        """
        批量执行测试用例
        :return: 返回测试用例结果[]
        """
        results = []

        log.info("开始执行任务")
        if self.parallel:
            threads = []
            for case in self.test_cases:
                t = MyThread(
                    func=run_testcase_script,
                    args=(case["case_file"], case["case_args"]),
                    kwargs=case["kwargs"],
                    func_name=case,
                )
                threads.append(t)
                t.start()
            for t in threads:
                data = t.get_result()
                result = data[0]
                case_info = data[1]

                results.append(t.get_result()[0])
                log.info(f"测试报告：{t.get_result()}")

        else:
            for case in self.test_cases:
                try:
                    result = run_testcase_script(
                        case["case_file"], case["task"], case["suite"], case["case_args"], **case["kwargs"]
                    )
                    if result is not None:
                        results.append(result)
                        log.info(f"{case['case_file']} 测试报告：{result.report_path}")
                except:
                    log.info(f"{case['case_file']} 执行失败：{traceback.format_exc()}")
        try:
            report_path = create_overview_report(
                project_home=self.case_home.split("testcases")[0],
                report_name=self.suite_name,
                results=results,
            )
        except:
            report_path = ""
        total_case = 0
        pass_case = 0
        fail_case = 0
        for item in results:
            total_case += item.pass_case + item.fail_case
            pass_case += item.pass_case
            fail_case += item.fail_case
        return {
            "results": results,
            "suite_name": self.suite_name,
            "author": self.author,
            "total_case": total_case,
            "pass_case": pass_case,
            "fail_case": fail_case,
            "report_path": report_path,
        }



# -*- coding: utf-8 -*-
# @Time : 2023/07/04
# @Author : chengwenping2
# @File : __init__.py.py
"""
文件说明：
"""
__title__ = 'pig_sdk'
__description__ = '💯一款简洁、好用的接口自动化框架.'
__url__ = 'https://git.oak.net.cn/test_group/pigs_sdk/-/tree/master/pig_frame'
__version__ = "1.3.0",
__author__ = 'chengwenping2'
__author_email__ = 'chengwenping2@newhope.cn'
__license__ = 'MIT'
__copyright__ = 'Copyright 2023 股份总部-数字科技大部-质量技术中心'




# -*- coding: utf-8 -*-
# @Time : 2023/7/7
# @Author : chengwenping2
"""
文件说明：批量执行测试用例demo，执行脚本必须位于testcases目录下，可按需包装
"""
from pig_frame.decorators import pig_frame_suite


@pig_frame_suite(suite_name="测试集", creator="负责人")
def run_suite(test_suite, **kwargs):
    # 改成 True 可以并行执行用例
    test_suite.parallel = False
    test_suite.add_case("idl_demo.py", **kwargs)
    # test_suite.add_cases_by_folder(folder_path=f"", exclude=['系统'], kwargs=kwargs)
    # test_suite.add_cases_by_folder(folder_path=f"", kwargs=kwargs)


if __name__ == "__main__":
    # env_config_file:环境配置文件相对路径（相对于conf/），例如/conf/uat.json就写uat.json，/conf/product/test.json就写product/test.json
    run_suite(env_config_file='dev.json')



# -*- coding: utf-8 -*-
# @Time : 2023/07/04
# @Author : chengwenping2
# @File : testcase.py
"""
文件说明：
"""

from pig_frame.common import *
from decimal import Decimal


def contain(exp, actual, result=None, filter_keys=None, limit_keys=None):
    """
    判断实际值是否包含在期望值中
    @param exp: 期望值可以传入字符串、dict、list、及多层嵌套对象
    @param actual: 实际值可以传入字符串、dict、list、及多层嵌套对象
    @param result: 一般不需要传入，初始已有差异时，可传入list
    @param limit_keys: 通过jsonpath条件表达式限制数据范围
    @param filter_keys: 需要跳过比对的keys list传入，写法['response.data.*','response.data','response[*].data'],
         .*结尾表示跳过对象，[*]过滤list对象,[1]指定list对象中的第二个
    @return: 原组形式返回（true，result）
    """
    if result is None:
        result = []
    if filter_keys is None:
        filter_keys = []
    if limit_keys is None:
        limit_keys = []
    if isinstance(exp, dict) or isinstance(exp, list):
        if isinstance(actual, type(exp)):
            exp_key_list = dict_to_simple(exp)
            actual_key_list = dict_to_simple(actual)
            # 获取两个对象所有key的交集
            mix_list = [val for val in exp_key_list if val in actual_key_list]
            # 实际值中存在，期望值中不存在的key
            diff_list_actual = list(set(exp_key_list).difference(set(actual_key_list)))
            # 比较相同key的value值
            for mix_key in mix_list:
                if not limit_check_key(limit_keys, mix_key):
                    continue
                if filter_key(filter_keys, mix_key):
                    continue
                if obj_search(exp, mix_key) != obj_search(actual, mix_key):
                    result = result + compare_number(
                        obj_search(exp, mix_key),
                        obj_search(actual, mix_key),
                        key=mix_key,
                    )
            for diff_actual_key in diff_list_actual:
                if not limit_check_key(limit_keys, diff_actual_key):
                    continue
                if filter_key(filter_keys, diff_actual_key):
                    continue
                result.append(f"{diff_actual_key} => 在实际值中不存在")
        else:
            result.append(f"期望值与实际值类型不匹配")
            return {
                "check_result": False,
                "check_detail": result,
                "check_type": "contain",
                "expect": exp,
                "actual": actual,
                "filter_keys": filter_keys,
                "limit_keys": limit_keys,
            }
    else:
        if str(actual).__contains__(str(exp)):
            return {
                "check_result": True,
                "check_detail": result,
                "check_type": "contain",
                "expect": exp,
                "actual": actual,
                "filter_keys": filter_keys,
                "limit_keys": limit_keys,
            }
        else:
            result.append(f"实际值 {actual} 中不存在 {exp}")
            return {
                "check_result": False,
                "check_detail": result,
                "check_type": "contain",
                "expect": exp,
                "actual": actual,
                "filter_keys": filter_keys,
                "limit_keys": limit_keys,
            }
    if len(result) == 0:
        return {
            "check_result": True,
            "check_detail": result,
            "check_type": "contain",
            "expect": exp,
            "actual": actual,
            "filter_keys": filter_keys,
            "limit_keys": limit_keys,
        }
    else:
        return {
            "check_result": False,
            "check_detail": result,
            "check_type": "contain",
            "expect": exp,
            "actual": actual,
            "filter_keys": filter_keys,
            "limit_keys": limit_keys,
        }


def not_contain(exp, actual, result=None, filter_keys=None, limit_keys=None):
    """
    判断实际值是不包含在期望值中
    @param exp: 期望值可以传入字符串、dict、list、及多层嵌套对象
    @param actual: 实际值可以传入字符串、dict、list、及多层嵌套对象
    @param result: 一般不需要传入，初始已有差异时，可传入list
     @param filter_keys: 需要跳过比对的keys list传入，写法['response.data.*','response.data','response[*].data'],
         .*结尾表示跳过对象，[*]过滤list对象,[1]指定list对象中的第二个
    @return: 原组形式返回（true，result）
    @param limit_keys: 通过jsonpath条件表达式限制数据范围
    """
    if result is None:
        result = []
    if filter_keys is None:
        filter_keys = []
    if limit_keys is None:
        limit_keys = []
    if isinstance(exp, dict) or isinstance(exp, list):
        if isinstance(actual, type(exp)):
            exp_key_list = dict_to_simple(exp)
            actual_key_list = dict_to_simple(actual)
            # 获取两个对象所有key的交集
            mix_list = [val for val in exp_key_list if val in actual_key_list]
            # 实际值中存在，期望值中不存在的key
            diff_list_actual = list(set(exp_key_list).difference(set(actual_key_list)))
            # 比较相同key的value值
            for mix_key in mix_list:
                if not limit_check_key(limit_keys, mix_key):
                    continue
                if filter_key(filter_keys, mix_key):
                    continue
                if obj_search(exp, mix_key) != obj_search(actual, mix_key):
                    result = result + compare_number(
                        obj_search(exp, mix_key),
                        obj_search(actual, mix_key),
                        key=mix_key,
                    )
            for diff_actual_key in diff_list_actual:
                if not limit_check_key(limit_keys, diff_actual_key):
                    continue
                if filter_key(filter_keys, diff_actual_key):
                    continue
                result.append(f"{diff_actual_key} => 在实际值中不存在")
        else:
            result.append(f"期望值与实际值类型不匹配")
            return {
                "check_result": False,
                "check_detail": result,
                "check_type": "not_contain",
                "expect": exp,
                "actual": actual,
                "filter_keys": filter_keys,
                "limit_keys": limit_keys,
            }
    else:
        if str(actual).__contains__(str(exp)):
            return {
                "check_result": False,
                "check_detail": result,
                "check_type": "not_contain",
                "expect": exp,
                "actual": actual,
                "filter_keys": filter_keys,
                "limit_keys": limit_keys,
            }
        else:
            result.append(f"实际值 {actual} 中不存在 {exp}")
            return {
                "check_result": True,
                "check_detail": result,
                "check_type": "not_contain",
                "expect": exp,
                "actual": actual,
                "filter_keys": filter_keys,
                "limit_keys": limit_keys,
            }
    if len(result) == 0:
        return {
            "check_result": False,
            "check_detail": result,
            "check_type": "not_contain",
            "expect": exp,
            "actual": actual,
            "filter_keys": filter_keys,
            "limit_keys": limit_keys,
        }
    else:
        return {
            "check_result": True,
            "check_detail": result,
            "check_type": "not_contain",
            "expect": exp,
            "actual": actual,
            "filter_keys": filter_keys,
            "limit_keys": limit_keys,
        }


def equal_no_order(exp: list or set, actual: list or set):
    """
    两个可迭代对象对比，list 或者 set

    重复元素会自动去重，使用时请保证入参没有重复数据
    @param actual: list or set实际结果
    @param exp: list or set 期望对象
    """
    result = []
    _exp = exp
    _actual = actual
    if isinstance(exp, list):
        _exp = set(exp)
    if isinstance(actual, list):
        _actual = set(actual)

    _diff = list(_exp.difference(_actual))
    if len(_diff) == 0:
        return {
            "check_result": True,
            "check_detail": result,
            "check_type": "equal",
            "expect": exp,
            "actual": actual,
            "filter_keys": [],
            "limit_keys": [],
        }
    else:
        result.append(f"{_diff} => 在实际值中不存在")
        return {
            "check_result": False,
            "check_detail": result,
            "check_type": "equal",
            "expect": exp,
            "actual": actual,
            "filter_keys": [],
            "limit_keys": [],
        }


def equal(exp, actual, result=None, filter_keys=None, limit_keys=None):
    """
    通用比对方法，验证是否相等
    @param result: list
    @param actual: object
    @param exp: object
    @param limit_keys: 通过jsonpath条件表达式限制数据范围
    @param filter_keys: 需要跳过比对的keys list传入，写法['response.data.*','response.data','response[*].data'],
         .*结尾表示跳过对象，[*]过滤list对象,[1]指定list对象中的第二个
    @return (True,result) 元组形式返回结果0是比对结果，1详细差异
    """
    if result is None:
        result = []
    if filter_keys is None:
        filter_keys = []
    if limit_keys is None:
        limit_keys = []
    if isinstance(exp, dict) or isinstance(exp, list):
        if isinstance(actual, type(exp)):
            exp_key_list = dict_to_simple(exp)
            actual_key_list = dict_to_simple(actual)
            # 获取两个对象所有key的交集
            mix_list = [val for val in exp_key_list if val in actual_key_list]
            # 期望值中存在，实际值中不存在的key
            diff_list_exp = list(set(exp_key_list).difference(set(actual_key_list)))
            # 实际值中存在，期望值中不存在的key
            diff_list_actual = list(set(actual_key_list).difference(set(exp_key_list)))
            # 比较相同key的value值
            for mix_key in mix_list:
                if not limit_check_key(limit_keys, mix_key):
                    continue
                if filter_key(filter_keys, mix_key):
                    continue
                if obj_search(exp, mix_key) != obj_search(actual, mix_key):
                    result = result + compare_number(
                        obj_search(exp, mix_key),
                        obj_search(actual, mix_key),
                        key=mix_key,
                    )
            for diff_exp_key in diff_list_exp:
                if not limit_check_key(limit_keys, diff_exp_key):
                    continue
                if filter_key(filter_keys, diff_exp_key):
                    continue
                result.append(f"{diff_exp_key} => 在实际值中不存在")
            for diff_actual_key in diff_list_actual:
                if not limit_check_key(limit_keys, diff_actual_key):
                    continue
                if filter_key(filter_keys, diff_actual_key):
                    continue
                result.append(f"{diff_actual_key} => 在期望值中不存在")
        else:
            result.append(f"期望值与实际值类型不匹配")
            return {
                "check_result": False,
                "check_detail": result,
                "check_type": "equal",
                "expect": exp,
                "actual": actual,
                "filter_keys": filter_keys,
                "limit_keys": limit_keys,
            }
    else:
        result = result + compare_number(exp, actual)
    if len(result) == 0:
        return {
            "check_result": True,
            "check_type": "equal",
            "check_detail": result,
            "expect": exp,
            "actual": actual,
            "filter_keys": filter_keys,
            "limit_keys": limit_keys,
        }
    else:
        return {
            "check_result": False,
            "check_type": "equal",
            "check_detail": result,
            "expect": exp,
            "actual": actual,
            "filter_keys": filter_keys,
            "limit_keys": limit_keys,
        }


def structure(
    exp, actual, result=None, filter_keys=None, limit_keys=None, check_value_type=False
):
    """
    通用比对方法，验证响应结构是否相同
    @param result: list
    @param actual: object
    @param exp: object
    @param limit_keys: 通过jsonpath条件表达式限制数据范围
    @param filter_keys: 需要跳过比对的keys list传入，写法['response.data.*','response.data','response[*].data'],
         .*结尾表示跳过对象，[*]过滤list对象,[1]指定list对象中的第二个
    @return (True,result) 元组形式返回结果0是比对结果，1详细差异
    @param check_value_type:是否需要比对字段类型
    """
    if result is None:
        result = []
    if filter_keys is None:
        filter_keys = []
    if limit_keys is None:
        limit_keys = []
    if type(exp) != type(actual):
        result.append(f"期望值: {exp} 与实际值: {actual} 类型不等")
    if isinstance(exp, dict) or isinstance(exp, list):
        if isinstance(actual, type(exp)):
            exp_key_list = dict_to_simple(exp)
            actual_key_list = dict_to_simple(actual)
            # 获取两个对象所有key的交集
            mix_list = [val for val in exp_key_list if val in actual_key_list]
            # 期望值中存在，实际值中不存在的key
            diff_list_exp = list(set(exp_key_list).difference(set(actual_key_list)))
            # 实际值中存在，期望值中不存在的key
            diff_list_actual = list(set(actual_key_list).difference(set(exp_key_list)))
            # 比较相同key的value值
            for mix_key in mix_list:
                if not limit_check_key(limit_keys, mix_key):
                    continue
                if filter_key(filter_keys, mix_key):
                    continue
                if (
                    type(obj_search(exp, mix_key)) != type(obj_search(actual, mix_key))
                    and check_value_type
                ):
                    result.append(
                        f"{mix_key} => 期望值: {obj_search(exp, mix_key)} 与实际值: "
                        f"{obj_search(actual, mix_key)} 类型不同"
                    )
            for diff_exp_key in diff_list_exp:
                if not limit_check_key(limit_keys, diff_exp_key):
                    continue
                if filter_key(filter_keys, diff_exp_key):
                    continue
                result.append(f"{diff_exp_key} => 在实际值中不存在")
            for diff_actual_key in diff_list_actual:
                if not limit_check_key(limit_keys, diff_actual_key):
                    continue
                if filter_key(filter_keys, diff_actual_key):
                    continue
                result.append(f"{diff_actual_key} => 在期望值中不存在")
        else:
            result.append(f"期望值与实际值类型不匹配")
            return {
                "check_result": False,
                "check_detail": result,
                "check_type": "structure",
                "expect": exp,
                "actual": actual,
                "filter_keys": filter_keys,
                "limit_keys": limit_keys,
            }
    else:
        if type(exp) != type(actual) and check_value_type:
            result.append(f"期望值: {exp} 与实际值: {actual} 类型不等")
    if len(result) == 0:
        return {
            "check_result": True,
            "check_type": "structure",
            "check_detail": result,
            "expect": exp,
            "actual": actual,
            "filter_keys": filter_keys,
            "limit_keys": limit_keys,
        }
    else:
        return {
            "check_result": False,
            "check_type": "structure",
            "check_detail": result,
            "expect": exp,
            "actual": actual,
            "filter_keys": filter_keys,
            "limit_keys": limit_keys,
        }


def not_equal(exp, actual, result=None, filter_keys=None, limit_keys=None):
    """
    通用比对方法，验证是否不等
    @param result: list
    @param actual: object
    @param exp: object
    @param limit_keys: 通过jsonpath条件表达式限制数据范围
    @param filter_keys: 需要跳过比对的keys list传入，写法['response.data.*','response.data','response[*].data'],
         .*结尾表示跳过对象，[*]过滤list对象,[1]指定list对象中的第二个
    @return (True,result) 元组形式返回结果0是比对结果，1详细差异
    """
    if result is None:
        result = []
    if filter_keys is None:
        filter_keys = []
    if limit_keys is None:
        limit_keys = []
    if isinstance(exp, dict) or isinstance(exp, list):
        if isinstance(actual, type(exp)):
            exp_key_list = dict_to_simple(exp)
            actual_key_list = dict_to_simple(actual)
            # 获取两个对象所有key的交集
            mix_list = [val for val in exp_key_list if val in actual_key_list]
            # 期望值中存在，实际值中不存在的key
            diff_list_exp = list(set(exp_key_list).difference(set(actual_key_list)))
            # 实际值中存在，期望值中不存在的key
            diff_list_actual = list(set(actual_key_list).difference(set(exp_key_list)))
            # 比较相同key的value值
            for mix_key in mix_list:
                if not limit_check_key(limit_keys, mix_key):
                    continue
                if filter_key(filter_keys, mix_key):
                    continue
                if obj_search(exp, mix_key) != obj_search(actual, mix_key):
                    result = result + compare_number(
                        obj_search(exp, mix_key),
                        obj_search(actual, mix_key),
                        key=mix_key,
                    )
            for diff_exp_key in diff_list_exp:
                if not limit_check_key(limit_keys, diff_exp_key):
                    continue
                if filter_key(filter_keys, diff_exp_key):
                    continue
                result.append(f"{diff_exp_key} => 在实际值中不存在")
            for diff_actual_key in diff_list_actual:
                if not limit_check_key(limit_keys, diff_actual_key):
                    continue
                if filter_key(filter_keys, diff_actual_key):
                    continue
                result.append(f"{diff_actual_key} => 在期望值中不存在")
        else:
            result.append(f"期望值与实际值类型不匹配")
            return {
                "check_result": True,
                "check_detail": result,
                "check_type": "not_equal",
                "expect": exp,
                "actual": actual,
                "filter_keys": filter_keys,
                "limit_keys": limit_keys,
            }
    else:
        result = result + compare_number(exp, actual)
    if len(result) == 0:
        return {
            "check_result": False,
            "check_type": "not_equal",
            "check_detail": result,
            "expect": exp,
            "actual": actual,
            "filter_keys": filter_keys,
            "limit_keys": limit_keys,
        }
    else:
        return {
            "check_result": True,
            "check_type": "not_equal",
            "check_detail": result,
            "expect": exp,
            "actual": actual,
            "filter_keys": filter_keys,
            "limit_keys": limit_keys,
        }


def expression(expression, expect="", actual=""):
    """
    自定义表达式断言
    :param expression:
    :return:
    """
    return {
        "check_result": bool(expression),
        "check_type": "expression",
        "check_detail": [],
        "expect": expect,
        "actual": actual,
        "filter_keys": [],
        "limit_keys": [],
    }


def compare_number(exp, actual, key=""):
    result = []
    try:
        if isinstance(exp, int) or isinstance(exp, float):
            if isinstance(actual, int) or isinstance(actual, float):
                if Decimal(str(exp)) != Decimal(str(actual)):
                    result.append(f"期望值:{key} {str(exp)} 与实际值: {str(actual)} 不等")
            else:
                if exp != actual:
                    result.append(
                        f"期望值:{key} {exp} {type(exp)}与实际值: {actual} {type(actual)}不等"
                    )
        else:
            if exp != actual:
                result.append(
                    f"期望值:{key} {exp} {type(exp)}与实际值: {actual} {type(actual)}不等"
                )
    except:
        if exp != actual:
            result.append(f"期望值:{key} {exp} {type(exp)}与实际值: {actual} {type(actual)}不等")
    return result


if __name__ == "__main__":
    data = {
        "responseI18nParam": None,
        "pageNum": 1,
        "pageSize": 10,
        "totalCount": 17,
        "dataList": [
            {
                "success": True,
                "statusCodeEnum": "SUCCESS",
                "statusCode": "0000",
                "messageCode": None,
                "message": None,
                "exception": None,
                "requestI18nParam": None,
            }
        ],
        "pageCount": 2,
    }
    data1 = {
        "responseI18nParam": None,
        "pageNum": 1,
        "pageSize": 10,
        "totalCount": 45,
        "dataList": [
            {
                "success": True,
                "statusCodeEnum": "SUCCESS",
                "statusCode": "00500",
                "messageCode": None,
                "message": None,
                "exception": None,
                "requestI18nParam": None,
            },
            {
                "success": True,
                "statusCodeEnum": "SUCCESS",
                "statusCode": "0000",
                "messageCode": None,
                "message": None,
                "exception": None,
                "requestI18nParam": None,
            },
        ],
        "pageCount": 2,
    }
    result = equal(exp=data, actual=data1, limit_keys=["dataList[0]"])
    print(result)



# -*- coding: utf-8 -*-
# @Time : 2023/07/04
# @Author : chengwenping2
# @File : http_request.py
"""
文件说明：http请求执行器，
"""
import re
import traceback

import pyjson5
import requests
import urllib3
from bs4 import BeautifulSoup

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

from datetime import datetime
from requests.adapters import HTTPAdapter
from pig_frame import log
from pig_frame.exceptions import PigFrameCaseException
from pig_frame.ddt import process_mock_data


def request(method, url, description="", retry_times=1, **kwargs):
    """
    http请求统一处理，请求参数、响应、状态码等统一返回，请求网络异常时，间隔5S重试三次
    :param url:
    :param method:支持get、post、等常用请求方式
    :param description:接口描述
    :param retry_times:重试次数
    :param kwargs:扩展参数支持"headers", "cookies", "params", "data", "json"
    :param files 标准写法
    {
    "field1" : ("filename1", open("filePath1", "rb")),
    "field2" : ("filename2", open("filePath2", "rb"), "image/jpeg"),
    "field3" : ("filename3", open("filePath3", "rb"), "image/jpeg", {"refer" :"localhost"})
    }
    :return:
    """
    for key in kwargs:
        if key not in [
            "headers",
            "cookies",
            "params",
            "data",
            "json",
            "files",
            "auth",
            "timeout",
            "allow_redirects",
            "proxies",
            "hooks",
            "stream",
            "verify",
            "cert",
        ]:
            raise Exception(f"传入参数 {key} 不支持")
    for key in ["params", "data", "json"]:
        if kwargs.get(key) is not None:
            kwargs[key] = process_mock_data(kwargs[key])
    start_time = datetime.now()

    try:
        result = _request(method, url, description, retry_times, **kwargs)
    except Exception as e:
        raise PigFrameCaseException(
            description=description,
            case_name=str(e),
            check_detail=traceback.format_exc(limit=-1, chain=False),
            use_time=(datetime.now() - start_time).microseconds / 1000,
        )

    if result is None:
        raise PigFrameCaseException(
            description=description,
            case_name="接口返回结果为空",
            check_detail="接口返回结果为空",
            use_time=(datetime.now() - start_time).microseconds / 1000,
        )
    return result


def _request(method, url, description="", retry_times=1, **kwargs):
    s = requests.Session()
    s.mount("http://", HTTPAdapter(max_retries=retry_times))
    s.mount("https://", HTTPAdapter(max_retries=retry_times))

    start_time = datetime.now()
    r = s.request(
        method=str(method).lower(),
        # 调用方需要将完整 url 拆分为 host+path传递，且 host 包含协议部分
        url=url,
        # allow_redirects=kwargs.pop("allow_redirects", False),
        # timeout=kwargs.pop("timeout", 15),
        # verify=kwargs.pop("verify", False),
        # 扩展参数直接对外提供，调用方传递需要了解 requests 库的文档，将其组装为 json 格式通过入参 json 传递进来
        **kwargs,
    )
    use_time = (datetime.now() - start_time).microseconds / 1000
    result = None
    if r is not None:
        html_etree = None
        jsonp = None
        try:
            response = r.json()
            try:
                jsonp = pyjson5.loads(re.findall("\(.*\)", r.text)[0][1:-1])
            except:
                pass
        except:
            response = r.text
            try:
                html_etree = BeautifulSoup(r.text, 'html.parser')
            except:
                pass

        result = {
            "params": {
                "params": kwargs.get("params"),
                "data": kwargs.get("data"),
                "json": kwargs.get("json"),
            },
            "response": response,
            "jsonp": jsonp,
            "html_tree": html_etree,
            "status_code": r.status_code,
            "url": r.url,
            "path": r.request.path_url,
            # 请求耗时，单位 ms
            "use_time": use_time,
            "description": description,
            "headers": r.request.headers,
            "response_headers": r.headers,
            "method": str(method).upper(),
            "response_time": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        }

        log.info(f"请求地址：{url}，请求入参：{result.get('params')}")
        log.info(f"请求响应：{response}")
        log.info(f"请求response_headers：{r.headers}")

    return result



# -*- coding: utf-8 -*-
# @Time : 2023/07/04
# @Author : chengwenping2
# @File : my_thread.py
"""
文件说明：多线程执行任务，并获取返回
"""
import threading


class MyThread(threading.Thread):
    def __init__(self, func, args=(), kwargs=None, func_name=""):
        super(MyThread, self).__init__()
        self.func = func
        self.args = args
        self.kwargs = kwargs
        self.result = None
        self.func_name = func_name

    def run(self):
        if self.kwargs is not None:
            self.result = self.func(*self.args, **self.kwargs)
        else:
            self.result = self.func(*self.args)

    def get_result(self):
        threading.Thread.join(self)  # 等待线程执行完毕
        try:
            return self.result, self.func_name
        except Exception:
            return None



# -*- coding: utf-8 -*-
# @Time : 2023/07/04
# @Author : chengwenping2
import os
import sys
import logging
import time
import traceback
from datetime import datetime

from pig_frame import log
from pig_frame.common import (
    case_to_xmind,
    remove_log,
    merge_dict_with_self_variables,
    get_call_link,
)
from pig_frame.exceptions import PigFrameCaseException, PigFrameCaseStopException
from pig_frame.test_case import TestCase
from pig_frame.test_suite import TestSuite


def execute_time(func):
    """
    打印方法执行时间
    @param func:
    @return:
    """

    def wrapper(*args, **kwargs):
        start = time.time()
        logging.info(f"{args} {kwargs}")
        func_return = func(*args, **kwargs)
        end = time.time()
        print(f"{func.__name__} 执行耗时 {end - start} 秒")
        logging.info(f"{func.__name__} 执行耗时 {end - start} 秒")
        return func_return

    return wrapper


def check_position(func):
    """
    检测当前路径是否在pig_frame标准项目下
    @param func:
    @return:
    """

    def check_path(*args, **kwargs):
        if check_pig_frame_path():
            return func(*args, **kwargs)
        else:
            print(
                "请在符合pig_frame框架标准目录结构的路径中执行此命令\n\n"
                "提示：包含idl/、testcases/的目录\n"
            )
            sys.exit(1)

    return check_path


def check_pig_frame_path():
    """
    检测是是否在testcases下执行
    @return:
    """
    path = os.getcwd()
    if os.path.exists(path):
        files = os.listdir(path)
        if "testcases" in files:
            return True

    return False


def pig_frame_case(
    report_name, creator, unique_report=False, create_xmind=False, output_json_file=True
):
    """
    pig_frame case 入口
    @param report_name:报告名称
    @param creator:创建人
    @param unique_report: True 多次执行报告覆盖、Fasle 多次执行报告不覆盖
    @param create_xmind:True 生成xmind 文件
    @param output_json_file:True 生成json数据报告
    @return:
    """

    def decorator_name(func):
        def decorated(**kwargs):
            start_time = datetime.now()
            path = kwargs.get("file_abs_path_especially", get_call_link(step=1))
            remove_log(path)
            kwargs["file_abs_path_especially"] = path
            suite = kwargs.get("suite")
            task = kwargs.get("task")

            test_case = TestCase(
                report_name=report_name,
                creator=creator,
                output_json_file=output_json_file,
            )
            try:
                kwargs = merge_dict_with_self_variables(path, kwargs.get("env_config_file", "test.json"), **kwargs)
                func(test_case, **kwargs)
            except PigFrameCaseException as e:
                log.error(f"{report_name} 执行失败：" + traceback.format_exc())
                test_case.check_without_request(
                    description=e.description,
                    case_name=e.case_name,
                    case_assert={
                        "check_result": False,
                        "check_detail": e.check_detail,
                    },
                    use_time=e.use_time,
                )
            except PigFrameCaseStopException as e:
                log.info(f"{report_name} 执行结束：" + e.stop_reason)
            except Exception as e:
                log.error(f"{report_name} 执行失败：" + traceback.format_exc())
                test_case.check_without_request(
                    description=f"{report_name}",
                    case_name=str(e),
                    case_assert={
                        "check_result": False,
                        "check_detail": traceback.format_exc(),
                    },
                    use_time=(datetime.now() - start_time).microseconds / 1000,
                )
            finally:
                test_case.output_report(path, unique=unique_report, suite=suite, task=task)
                if create_xmind:
                    case_to_xmind(
                        report=test_case.report,
                        report_name=test_case.report_name,
                        creator=test_case.creator,
                        path=os.path.dirname(test_case.report_path),
                    )
                log.info(f"测试用例 {report_name} 已执行完毕.")
                return test_case

        return decorated

    return decorator_name


def pig_frame_suite(suite_name, creator):
    """
    测试集入口
    @param suite_name:测试集名称
    @param creator:创建人
    @return:
    """

    def decorator_name(func):
        def decorated(**kwargs):
            if kwargs.get("file_abs_path_especially") is not None:
                path = kwargs.pop("file_abs_path_especially")
            else:
                path = get_call_link(step=1)

            remove_log(path)
            task_name = kwargs.get("task_name")
            if task_name:
                remove_log(task_name)

            if 'env_config_file' in kwargs.keys():
                env_config_file = kwargs.pop('env_config_file')
                kwargs = merge_dict_with_self_variables(path, env_config_file, **kwargs)
                kwargs["suite_env_config_file"] = env_config_file

            test_suite = TestSuite(
                task_name=kwargs.get("task_name"),
                suite_name=kwargs.pop("suite_name", suite_name),
                case_home=os.path.dirname(path),
                author=creator,
            )
            try:
                func(test_suite, **kwargs)
                return test_suite.run_cases()
            except:
                log.error(f"测试集执行失败{suite_name}：" + traceback.format_exc())
                return None

        return decorated

    return decorator_name





# -*- coding: utf-8 -*-
# @Time : 2023/07/04
# @Author : chengwenping2

def calc_rate(pass_case, fail_case):
    if pass_case + fail_case != 0:
        rate = round((pass_case / (pass_case + fail_case)) * 100, 2)
        rate = "%.2f" % rate
    else:
        rate = 0
    return rate



# -*- coding: utf-8 -*-
# @Time : 2023/07/05
# @Author : chengwenping2
# @File    : common
# @Description :初始化数据~
import copy
import hashlib
import logging
import os.path
import traceback
import time
import jmespath
import pyjson5

from pig_frame.common import dict_to_simple, get_log_conf
from pig_frame.utils import tail_f, tail


def execute_time(func):
    """
    打印方法执行时间
    @param func:
    @return:
    """

    def wrapper(*args, **kwargs):
        start = time.time()
        logging.info(f"{args} {kwargs}")
        func_return = func(*args, **kwargs)
        end = time.time()
        print(f"{func.__name__} 执行耗时 {end - start} 秒")
        logging.info(f"{func.__name__} 执行耗时 {end - start} 秒")
        return func_return

    return wrapper

def get_user_path():
    """
    获取全局文件路径
    @return:
    """
    if not os.path.exists(f"{os.path.expanduser('~')}{os.sep}pig_frame"):
        os.mkdir(f"{os.path.expanduser('~')}{os.sep}pig_frame")
    return f"{os.path.expanduser('~')}{os.sep}pig_frame"


def md5(string):
    m = hashlib.md5()
    m.update(string.encode("utf-8"))
    return m.hexdigest()


def str_hash(string):
    return md5(string)


def batch_str_hash(stings):
    result = []
    for item in stings:
        result.append(str_hash(item))
    return result


def get_file_name(file_path):
    """
    获取文件文件名称
    @param file_path:
    @return:
    """
    return os.path.basename(file_path)


def get_python_files(file_path, result=[]):
    """
    获取指定目录下所有Python脚本绝对路径
    @param file_path:
    @param result:
    @return:
    """
    files = os.listdir(file_path)
    for file in files:
        new_path = file_path + os.sep + file
        if os.path.isdir(new_path):
            get_python_files(new_path, result)
        else:
            if file.endswith(".py") or file.endswith(".PY"):
                if new_path.__contains__("testcases"):
                    result.append(os.path.abspath(new_path))
    return result


def get_directory_tree(file_path, exclude_system_check=False):
    result = []
    files = os.listdir(file_path)
    files.sort()
    for file in files:
        new_path = file_path + os.sep + file
        if exclude_system_check:
            if (
                str(new_path).__contains__("系统检测")
                or str(new_path).__contains__("测试场景")
                or str(new_path).__contains__("测试物料")
            ):
                continue
        if os.path.isdir(new_path) and file != "__pycache__":
            result.append(
                {
                    "label": file,
                    "path": os.path.abspath(new_path),
                    "children": get_directory_tree(
                        new_path, exclude_system_check=exclude_system_check
                    ),
                }
            )
    return result

def read_conf(conf_path):
    try:
        with open(conf_path, "r", encoding="utf-8") as file:
            conf = pyjson5.loads(file.read())
            return conf
    except:
        return {}


def get_case_type(case_path):
    """
    获取测试用例类型
    @param case_path:
    @return:
    """
    try:
        with open(case_path, "r", encoding="utf-8") as file:
            lines = file.read()
            if lines.__contains__("TestSuite") or lines.__contains__("@pig_frame_suite"):
                return "用例集"
            elif lines.__contains__("TestCase") or lines.__contains__("@pig_frame_case"):
                if str(case_path).__contains__("测试场景"):
                    return "测试场景"
                elif str(case_path).__contains__("测试物料"):
                    return "测试物料"
                elif str(case_path).__contains__("服务点名"):
                    return "服务点名"
                elif str(case_path).__contains__("系统检测"):
                    return "系统检测"
                return "用例"
            else:
                return "未知"
    except:
        return "未知"


def count_case_check(case_path):
    total_case = 0
    try:
        with open(case_path, "r", encoding="utf-8") as file:
            lines = file.readlines()
            for line in lines:
                if str(line).__contains__("test_case.check"):
                    total_case += 1
    finally:
        return total_case


def count_models_status(directory_tree_child, status):
    """
    统计目录状态数据
    @param directory_tree_child:
    @param status:
    @return:
    """
    keys = dict_to_simple(directory_tree_child)
    count = 0
    for key in keys:
        if not str(key).endswith(".model_status"):
            continue
        if jmespath.search(key, directory_tree_child) == status:
            count += 1
    return count


def sorted_children(data):
    result = []
    data_bak = copy.deepcopy(data)
    try:
        for item in data:
            if len(item.get("children")) == 0:
                result.append(item)
        for item in data:
            if len(item.get("children")) != 0:
                result.append(item)
        return result
    except:
        logging.info(f"排序异常：{traceback.format_exc()}")
        return data_bak


# 数据统计效率优化


def filter_cases_by_directory_path(cases, directory_path):
    """查找给定目录的子项"""
    case_result = []
    for case in cases:
        if str(case["case_path"]).__contains__(directory_path):
            case_result.append(case)
    return case_result


def filter_key_by_end(keys, end):
    """
    找出给定list 以 end 结束的数据
    @param keys:
    @param end:
    @return:
    """
    result = []
    for key in keys:
        if str(key).endswith(end):
            result.append(key)
    return result


def filter_case_info_by_key(case_info, key, value):
    """
    通过给定key 和value 过滤数据
    @param case_info:
    @param key:
    @param value:
    @return:
    """
    result = []
    for case in case_info:
        if case[key] == value:
            result.append(case)
    return result


def sum_case_info(case_info, key):
    """
    通过给定key 汇总计算
    @param case_info:
    @param key:
    @return:
    """
    result = 0
    for case in case_info:
        if case["status"] == "" and key == "total_case":
            result += case["static_total_case"]
        else:
            if case[key] is not None:
                result += int(case[key])
    return result


def count_model(case_info):
    """
    统计根节点目录个数
    @param case_info:
    @return:
    """
    result = []
    for case in case_info:
        try:
            result.append(os.path.dirname(case["case_path"]))
        except:
            continue
    result = list(set(result))
    return result


def count_model_by_status(case_info):
    """
    统计各个根目录状态
    @param case_info:
    @return:
    """
    models = {}
    for case in case_info:
        try:
            model_path = os.path.dirname(case["case_path"])
            if model_path in models:
                models[model_path].append(case["case_result"])
            else:
                models[model_path] = [case["case_result"]]
        except:
            continue
    pass_model = 0
    fail_model = 0
    unexecuted = 0
    for key in models:
        all_status = list(set(models[key]))
        if "失败" in all_status or "部分通过" in all_status:
            fail_model += 1
        elif len(all_status) == 1 and all_status[0] == "通过":
            pass_model += 1
        else:
            unexecuted += 1

    return pass_model, fail_model, unexecuted


# 废弃，严重影响接口响应时间
def update_directory_tree_model_info(directory_tree):
    """
    更新models状态相关信息到目录树，同时对目录结构排序，子级节点的放前面
    @param directory_tree:
    @return:
    """
    if isinstance(directory_tree, dict):
        directory_tree["children"] = sorted_children(directory_tree["children"])
        directory_tree["models_unexecuted"] = count_models_status(
            directory_tree["children"], "未执行"
        )
        directory_tree["models_pass"] = count_models_status(
            directory_tree["children"], "通过"
        )
        directory_tree["models_part_pass"] = count_models_status(
            directory_tree["children"], "部分通过"
        )
        directory_tree["models_fail"] = count_models_status(
            directory_tree["children"], "失败"
        )

    if (
        directory_tree.get("children") is not None
        and len(directory_tree.get("children")) > 0
    ):
        for i in range(len(directory_tree.get("children"))):
            update_directory_tree_model_info(directory_tree.get("children")[i])
    return directory_tree


def update_directory_tree_cass_info(directory_tree, cases, show_all=True):
    """
    更新case相关信息到目录树
    @param directory_tree:
    @param cases:
    @return:
    """
    if isinstance(directory_tree, dict):
        if len(directory_tree["children"]) == 0:
            directory_tree["label"] = "" + directory_tree["label"]
        case_info = filter_cases_by_directory_path(cases, directory_tree["path"])
        directory_tree["script_total"] = len(case_info)
        models_count = len(count_model(case_info))
        directory_tree["models_count"] = models_count
        model_status = count_model_by_status(case_info)
        directory_tree["models_pass"] = model_status[0]
        directory_tree["models_fail"] = model_status[1]
        directory_tree["models_unexecuted"] = model_status[2]
        if model_status[0] == models_count:
            directory_tree["model_status"] = "通过"
        elif model_status[0] != 0 and model_status[1] != 0:
            directory_tree["model_status"] = "部分通过"
        elif model_status[0] == 0 and model_status[1] != 0:
            directory_tree["model_status"] = "失败"
        elif model_status[2] == models_count:
            directory_tree["model_status"] = "未执行"
            if not show_all:
                directory_tree.pop("children")
        else:
            directory_tree["model_status"] = "失败"
        directory_tree["script_finish"] = len(
            filter_case_info_by_key(case_info, "status", "完成")
        )
        directory_tree["script_pass"] = len(
            filter_case_info_by_key(case_info, "case_result", "通过")
        )
        directory_tree["script_fail"] = (
            directory_tree["script_finish"] - directory_tree["script_pass"]
        )
        directory_tree["script_running"] = len(
            filter_case_info_by_key(case_info, "status", "执行中")
        )
        directory_tree["script_unexecuted"] = len(
            filter_case_info_by_key(case_info, "status", "")
        )
        directory_tree["total_case"] = sum_case_info(case_info, "total_case")
        directory_tree["pass_case"] = sum_case_info(case_info, "pass_case")
        directory_tree["fail_case"] = sum_case_info(case_info, "fail_case")
        directory_tree["cases"] = case_info
    if (
        directory_tree.get("children") is not None
        and len(directory_tree.get("children")) > 0
    ):
        children_len = len(directory_tree.get("children"))
        for i in range(children_len):
            update_directory_tree_cass_info(
                directory_tree.get("children")[children_len - i - 1], cases, show_all
            )
            if (
                directory_tree.get("children")[children_len - i - 1].get("model_status")
                == "未执行"
                or directory_tree.get("children")[children_len - i - 1].get(
                    "models_count"
                )
                == 0
            ) and not show_all:
                directory_tree.get("children").pop(children_len - i - 1)
    return directory_tree


def tail_f_all_log(project_home, callback, interval=0.01):
    """
    读取 all.log最后一行，调用 callback 方法
    :param project_home: 项目根目录
    :param callback: 每读到一行新的日志，执行的方法
    :param interval: 读取时间间隔，默认 100ms
    :return:
    """
    log_path = project_home + "/logs/all.log"
    if not os.path.exists(log_path):
        project_home = get_log_conf(project_home)

    log_path = project_home + "/logs/all.log"
    tail_f(log_path, callback, interval)


def tail_all_log(project_home, callback, n=10):
    """
    读取 all.log最后n行，调用 callback 方法
    :param n: 读取的行数
    :param project_home: 项目跟目录
    :param callback: 每读到一行新的日志，执行的方法
    :return:
    """
    log_path = project_home + "/logs/all.log"
    if not os.path.exists(log_path):
        project_home = get_log_conf(project_home)

    log_path = project_home + "/logs/all.log"
    lines = tail(log_path, n)
    for line in lines:
        callback(line)


def get_case_log(case_path):
    """
    读取执行日志
    :return: 日志内容
    :param case_path 调用方脚本名称
    """
    try:
        file_name = os.path.basename(case_path).split(".")[0]
        project_home = case_path.split("/testcases/")[0]
        log_file = f"{project_home}/logs/{file_name}.log"
        if not os.path.exists(log_file):
            project_home = get_log_conf(project_home)

        log_file = f"{project_home}/logs/{file_name}.log"
        if not os.path.exists(log_file):
            return "日志文件不存在"

        with open(log_file, "r", encoding="utf-8") as f:
            return f.read()
    except:
        logging.error(f"获取日志异常：{traceback.format_exc()}")
        return traceback.format_exc()


if __name__ == "__main__":
    print(
        get_directory_tree(
            "/Users/tianmaocheng/ypsh/python_project/sgc_autotest/testcases/B2P"
        )
    )



# -*- coding: utf-8 -*-
# @Time : 2023/07/04
# @Author : chengwenping2
# @File : db.py
"""
文件说明：封装常用公共方法
"""

import pymysql
from sshtunnel import SSHTunnelForwarder
from pymysql import converters, FIELD_TYPE
import redis


conv = converters.conversions
conv[FIELD_TYPE.NEWDECIMAL] = float  # convert decimals to float
conv[FIELD_TYPE.DATE] = str  # convert dates to strings
conv[FIELD_TYPE.TIMESTAMP] = str  # convert dates to strings
conv[FIELD_TYPE.DATETIME] = str  # convert dates to strings
conv[FIELD_TYPE.TIME] = str  # convert dates to strings


def mysql_execute(**kwargs):
    """
    mysql 查询工具
    :param kwargs:
    :return:
    """
    ssh_host = kwargs.get("ssh_host")
    ssh_prot = kwargs.get("ssh_prot")
    ssh_user = kwargs.get("ssh_user")
    ssh_password = kwargs.get("ssh_password")

    host = kwargs.get("host")
    port = kwargs.get("port")
    db = kwargs.get("db")
    user = kwargs.get("user")
    password = kwargs.get("password")
    limit = kwargs.get("limit")
    sql = kwargs.get("sql")
    if limit is None:
        limit = 200

    server = SSHTunnelForwarder(
        ssh_address_or_host=(ssh_host, ssh_prot),
        ssh_username=ssh_user,
        ssh_password=ssh_password,
        remote_bind_address=(host, port))
    server.start()

    conn = pymysql.connect(host="localhost",
                         port=server.local_bind_port,
                         user=user,
                         password=password,
                         database=db,
                         charset='utf8',
                         connect_timeout=100,
                         cursorclass=pymysql.cursors.DictCursor,
                         conv=conv)

    cursor = conn.cursor()
    try:
        cursor.execute(sql)
    finally:
        cursor.close()
        conn.close()
    return cursor.fetchmany(limit)
def redis_execute(**kwargs):
    """
    redis查询工具
    :param kwargs:
    :return:
    """
    host = kwargs.get("host")
    port = kwargs.get("port")
    pw = kwargs.get("password")
    key = kwargs.get("key")
    conn = redis.Redis(
            host=host,
            port=port,
            password=pw,
            db=0
        )

    try:
        val = conn.get(key)
    finally:
        conn.close()
    return val




# -*- coding: utf-8 -*-
# @Time : 2023/7/4
# @Author : chengwenping2
# @File    : ddt
# @Description :数据驱动 ，数据处理方法

import os.path
import random
import re
import time

import openpyxl as openpyxl
import pyjson5
from faker import Faker
import uuid


# def get_excel_cases(path):
#     """
#     读取Excel 测试用例
#     @param path:Excel文件路径
#     @return:
#     """
#     excel_cases = []
#     if os.path.exists(path):
#         work_book = openpyxl.load_workbook(path)
#         sheets = work_book.get_sheet_names()
#         for sheet in sheets:
#             sheet_cases = work_book.get_sheet_by_name(sheet)
#             all_value = sheet_cases[f"A3:J{sheet_cases.max_row}"]
#             for row_data in all_value:
#                 try:
#                     if row_data[0].value == "是":
#                         excel_cases.append(
#                             {
#                                 "desc": row_data[1].value,
#                                 "case_name": row_data[2].value,
#                                 "path": row_data[3].value,
#                                 "method": str(row_data[4].value).upper(),
#                                 "params": process_data(row_data[5].value),
#                                 "data": process_data(row_data[6].value),
#                                 "body": process_data(row_data[7].value),
#                                 "assert_type": row_data[8].value,
#                                 "exp": process_data(row_data[9].value),
#                             }
#                         )
#                 except:
#                     continue
#     return excel_cases


def process_data(data):
    """
    处理请求参数，期望结果等 ，插入mock数据
    @param data:
    @return:
    """
    try:
        return pyjson5.loads(data)
    except:
        return data


def read_json_data(file_path):
    case_data = []
    try:
        with open(file_path, "r", encoding="utf-8") as file:
            temp_data = pyjson5.load(file.read())
            if isinstance(temp_data, list):
                case_data += temp_data
            elif isinstance(temp_data, dict):
                case_data.append(temp_data)
    except:
        pass
    return case_data


def get_json_data(file_path):
    """
    读取json文件数据
    @param file_path:
    @return:
    """
    case_data = []
    if str(file_path).__contains__("*"):
        dir_path = os.path.dirname(file_path)
        file_name = os.path.basename(file_path)
        file_name = str(file_name).replace("*", ".*")
        dir_files = os.listdir(dir_path)
        for file in dir_files:
            if re.match(file_name, file) is not None:
                case_data += read_json_data(f"{dir_path}{os.sep}{file}")

    else:
        case_data += read_json_data(file_path)
    return case_data


def process_mock_data(data, loacal="zh_CN"):
    try:
        return PigFrameMock(loacal).process_mock_data(data)
    except:
        return data


class PigFrameDdt:
    def __init__(self, file_path=None):
        self.excel_case = []
        self.case_data = []
        if file_path is not None:
            if str(file_path).lower().endswith(".xlsx"):
                self.excel_case = get_excel_cases(path=file_path)
            elif str(file_path).lower().endswith(".json"):
                self.case_data = get_json_data(file_path)


class PigFrameMock:
    def __init__(self, locale="zh_CN"):
        self.faker = Faker(locale=locale)

    def get_params(self, key):
        try:
            p1 = re.compile(r"[(](.*?)[)]", re.S)
            result = re.findall(p1, key)[0]
            return str(result).split(",")
        except:
            return []

    def process_mock_data(self, data):
        if isinstance(data, dict):
            for key in data:
                data[key] = self.process_mock_data(data[key])
        elif isinstance(data, list):
            for index, value in enumerate(data):
                data[index] = self.process_mock_data(value)
        else:
            return self.get_fake_vale(data)
        return data

    def get_fake_vale(self, key):
        if str(key).lower().startswith("@name"):
            return self.faker.name()
        elif str(key).lower().startswith("@address"):
            return self.faker.address()
        elif str(key).lower().startswith("@city_name"):
            return self.faker.city_name()
        elif str(key).lower().startswith("@country"):
            return self.faker.country()
        elif str(key).lower().startswith("@postcode"):
            return self.faker.postcode()
        elif str(key).lower().startswith("@street_address"):
            return self.faker.street_address()
        elif str(key).lower().startswith("@bank_country"):
            return self.faker.bank_country()
        elif str(key).lower().startswith("@bban"):
            return self.faker.bban()
        elif str(key).lower().startswith("@ean"):
            return self.faker.ean(length=13)
        elif str(key).lower().startswith("@ean13"):
            return self.faker.ean13()
        elif str(key).lower().startswith("@company"):
            return self.faker.company()
        elif str(key).lower().startswith("@credit_card_number"):
            return self.faker.credit_card_number(card_type=None)
        elif str(key).lower().startswith("@currency_code"):
            return self.faker.currency_code()
        elif str(key).lower().startswith("@currency_name"):
            return self.faker.currency_name()
        elif str(key).lower().startswith("@date"):
            pattern = self.get_params(key)
            if len(pattern) == 0:
                pattern = "%Y-%m-%d %H:%M:%S"
            else:
                pattern = pattern[0]
            return self.faker.date(pattern=pattern, end_datetime=None)
        elif str(key).lower().startswith("@file_name"):
            pattern = self.get_params(key)
            if len(pattern) == 0:
                pattern = None
            else:
                pattern = pattern[0]
            return self.faker.file_name(category=None, extension=pattern)
        elif str(key).lower().startswith("@email"):
            return self.faker.email()
        elif str(key).lower().startswith("@ipv4"):
            return self.faker.ipv4()
        elif str(key).lower().startswith("@ipv6"):
            return self.faker.ipv6()
        elif str(key).lower().startswith("@mac_address"):
            return self.faker.mac_address()
        elif str(key).lower().startswith("@url"):
            return self.faker.uri()
        elif str(key).lower().startswith("@first_name"):
            return self.faker.first_name()
        elif str(key).lower().startswith("@last_name"):
            return self.faker.last_name()
        elif str(key).lower().startswith("@phone_number"):
            return self.faker.phone_number()
        elif str(key).lower().startswith("@ssn"):
            pattern = self.get_params(key)
            if len(pattern) == 0:
                min_age = 18
                max_age = 90
            else:
                pattern = pattern[0]
                if pattern.__contains__("-"):
                    min_age = int(pattern.split("-")[0])
                    max_age = int(pattern.split("-")[1])
                else:
                    min_age = int(pattern)
                    max_age = int(pattern) + 1
            return self.faker.ssn(min_age=min_age, max_age=max_age)
        elif str(key).lower().startswith("@timestamp"):
            return str(int(time.time() * 1000))
        elif str(key).lower().startswith("@uuid"):
            return str(uuid.uuid4())
        elif str(key).lower().startswith("@trace_id"):
            return str(uuid.uuid4()).replace("-", "")
        elif str(key).lower().startswith("@dict"):
            return self.faker.pydict()
        elif str(key).lower().startswith("@float"):
            pattern = self.get_params(key)
            min_value = None
            max_value = None
            right_digits = 5
            if len(pattern) == 0:
                pattern = None
            else:
                if pattern[0].__contains__("-"):
                    min_value = int(pattern[0].split("-")[0])
                    max_value = int(pattern[0].split("-")[1])
                if len(pattern) == 2:
                    right_digits = int(pattern[1])
            return self.faker.pyfloat(
                right_digits=right_digits,
                positive=True,
                min_value=min_value,
                max_value=max_value,
            )
        elif str(key).lower().startswith("@int"):
            pattern = self.get_params(key)
            min_value = None
            max_value = None
            if pattern[0].__contains__("-"):
                min_value = int(pattern[0].split("-")[0])
                max_value = int(pattern[0].split("-")[1])
            return self.faker.pyint(min_value=min_value, max_value=max_value, step=1)
        elif str(key).lower().startswith("@list"):
            pattern = self.get_params(key)
            if len(pattern) == 0:
                pattern = 10
            else:
                pattern = int(pattern[0])
            return self.faker.pylist(nb_elements=pattern, variable_nb_elements=True)
        elif str(key).lower().startswith("@str"):
            pattern = self.get_params(key)
            min_value = None
            max_value = None
            if pattern[0].__contains__("-"):
                min_value = int(pattern[0].split("-")[0])
                max_value = int(pattern[0].split("-")[1])
            return self.faker.pystr(min_chars=min_value, max_chars=max_value)
        elif str(key).lower().startswith("@bool"):
            return random.choice([True, False])
        elif str(key).lower().startswith("@choice"):
            return random.choice(self.get_params(key))
        elif str(key).lower().startswith("@text"):
            pattern = self.get_params(key)
            if len(pattern) == 0:
                pattern = 10
            else:
                pattern = int(pattern[0])
            return self.faker.sentence(
                nb_words=pattern, variable_nb_words=True, ext_word_list=None
            )
        else:
            return key


if __name__ == "__main__":
    mock = PigFrameMock()
    print(mock.get_fake_vale("@ipv6"))
    print(mock.get_fake_vale("@mac_address"))
    print(mock.get_fake_vale("@url"))
    print(mock.get_fake_vale("@first_name"))
    print(mock.get_fake_vale("@last_name"))
    print(mock.get_fake_vale("@phone_number"))
    print(mock.get_fake_vale("@ssn(18-20)"))
    print(mock.get_fake_vale("@timestamp"))
    print(mock.get_fake_vale("@uuid"))
    print(mock.get_fake_vale("@trace_id"))
    print(mock.get_fake_vale("@float(5-9,6)"))
    print(mock.get_fake_vale("@str(10-20)"))
    print(mock.get_fake_vale("@bool"))
    print(mock.get_fake_vale("@choice(3,5,8)"))
    print(mock.get_fake_vale("@text"))



# -*- coding: utf-8 -*-
# @Time : 2023/7/4
# @Author : chengwenping2
import importlib
import os.path
import traceback
from pig_frame import log
from termcolor import colored
from pig_frame.common import check_case_path


def run_testcase_script(file, task, suite, env_config_file: str = None, **kwargs):
    """run specified testcase
    :param file 脚本路径
    :param env_config_file conf 目录下面对应的文件名称
    1.变量优先级处理
    3.运行脚本

    """
    try:
        check_case_path(file)

        module_path = os.path.abspath(file).replace(
            os.path.abspath(file).split("testcases")[0], ""
        )[0:-3]
        mod = importlib.import_module(f"{module_path.replace(os.sep,'.')}")
        # 执行用例
        mod_dict = mod.__dict__
        for key in mod_dict:
            if str(mod_dict[key]).__contains__(
                "pig_frame_case.<locals>.decorator_name.<locals>.decorated"
            ):
                kwargs["file_abs_path_especially"] = os.path.abspath(file)
                kwargs["suite"] = suite
                kwargs["task"] = task
                return getattr(mod, key)(env_config_file=env_config_file, **kwargs)
        if env_config_file is None:
            return getattr(mod, "idl_run_case")(**kwargs)
        else:
            return getattr(mod, "run_case")(env_config_file=env_config_file, **kwargs)
    except:

        log.info(colored(f"执行{file}发生错误  \n{traceback.format_exc()}", "red"))


def run_testcase_suite(file, task_name, env_config_file: str = None, **kwargs):
    """run specified testcase
    :param file 脚本路径
    :param env_config_file conf 目录下面对应的文件名称
    1.变量优先级处理
    3.运行脚本

    """
    try:
        check_case_path(file)

        module_path = os.path.abspath(file).replace(
            os.path.abspath(file).split("testcases")[0], ""
        )[0:-3]
        mod = importlib.import_module(f"{module_path.replace(os.sep,'.')}")
        importlib.reload(mod)
        mod_dict = mod.__dict__
        for key in mod_dict:
            if str(mod_dict[key]).__contains__(
                "pig_frame_suite.<locals>.decorator_name.<locals>.decorated"
            ):
                kwargs["file_abs_path_especially"] = os.path.abspath(file)
                kwargs["task_name"] = task_name
                return getattr(mod, key)(suite_env_config_file=env_config_file, **kwargs)
        # 执行用例
        if env_config_file is None:
            return getattr(mod, "run_suite")(kwargs)
        else:
            return getattr(mod, "run_suite")(suite_env_config_file=env_config_file, **kwargs)
    except Exception as e:
        log.info(colored(f"执行{file}发生错误{e}  \n{traceback.format_exc()}", "red"))
        return {
            "report_path": "执行异常",
            "pass_case": 0,
            "author": "未知",
            "total_case": 0,
            "fail_case": 0,
        }

if __name__ == "__main__":
    run_testcase_suite(
        ""
    )



# -*- coding: utf-8 -*-
# @Time : 2023/7/7
# @Author : chengwenping2

from pig_frame.asserts import *
from pig_frame.decorators import pig_frame_case
from pig_frame.common import *
from pig_frame.http_request import request


@pig_frame_case(report_name="用例名称", creator="负责人", unique_report=True)
def case_template(test_case, **kwargs):
    bar2 = kwargs.get("bar2")
    bar1 = kwargs.get("bar1")

    result = request(
        method="get",
        url="https://postman-echo.com/get",
        description="Postman Echo",
        json=None,
        data=None,
        params={"foo1": bar1, "foo2": bar2},
        headers=None,
    )

    test_case.check(
        result,
        case_name="Http 返回码为 200",
        case_assert=equal(200, jmespath.search("status_code", result)),
    )

    variable_name = jmespath.search("response.code", result)


if __name__ == "__main__":
    kwargs = {}
    # env_config_file:环境配置文件相对路径（相对于conf/），例如/conf/uat.json就写uat.json，/conf/product/test.json就写product/test.json
    case_template(env_config_file="test.json", **kwargs)



# -*- coding: utf-8 -*-
# @Time : 2023/7/4
# @Author : chengwenping2
# @File : scheduler.py
"""
文件说明：定时批量执行测试用例并发送测试报告
"""

import os
import sys
from datetime import datetime

from pig_frame.runner import run_testcase_suite
from pig_frame.se_common import get_case_type

from pig_frame import log
from pig_frame.common import get_project_home, report_overview
import pyjson5 as json5
import traceback
from pig_frame.mail import sent_mail
from pig_frame.task_job import Task
from pig_frame.test_suite import TestSuite


def write_report(report_name, content):
    report_path = ""
    try:
        project_home = get_project_home()
        if not os.path.exists(f"{project_home}{os.sep}test_report"):
            os.mkdir(f"{project_home}{os.sep}test_report")
        report_path = f'{project_home}{os.sep}test_report{os.sep}{report_name}.html'
        # report_path = f'{project_home}{os.sep}test_report{os.sep}{report_name}_{datetime.now().strftime("%Y-%m-%d_%H%M%S_%f")[:-3]}.html'
        log.info(f"{report_name}测试报告生成路径：{report_path}")
        f = open(report_path, "w", encoding="utf-8")
        f.write(content)
        f.close()
        return report_path
    except:
        return report_path


def run_cases(cases, task_detail):
    results = _run_cases(cases, task_detail)
    return results


def _run_cases(cases, task_detail):
    """
    通过定时任务执行用例，并收集用例执行结果发送报告
    :param cases: 用例列表
    :param task_detail: 配置的task详情
    :return:
    """
    begin_time = datetime.now()
    schedule_result = {}
    testcase = []
    testsuites = []
    for item in cases:
        if get_case_type(item.get("case_file")) == "用例集":
            testsuites.append(item)
        else:
            testcase.append(item)
    # 执行测试集
    suite_results = []
    task_name = task_detail.get("task_name")
    for suite in testsuites:
        try:
            kwargs = suite.get("kwargs")
            if kwargs is None:
                kwargs = {}
            suite_results = suite_results + run_testcase_suite(
                suite["case_file"], task_name, env_config_file=suite.get("case_args"), **kwargs
            ).get("results")

        except:
            continue
    # 执行测试用例
    test_suite = TestSuite()
    test_suite.test_cases = testcase
    results = test_suite.run_cases()

    # 汇总执行结果发送报告
    sum_results = results.get("results") + suite_results
    report_content = report_overview(sum_results, upload=False)
    report_name = task_detail.get("subject", "自动化用例执行报告")
    report_subject = task_detail.get("subject", "自动化用例执行报告") + datetime.now().strftime(
        "%Y-%m-%d_%H%M%S_%f")[:-3]

    # 更新执行记录
    suite_report_path = write_report(report_name, report_content[0])
    sum_pass = 0
    sum_fail = 0

    for sub_result in sum_results:
        sum_pass += sub_result.pass_case
        sum_fail += sub_result.fail_case

    if sum_fail > 0:
        report_subject = "[异常]" + report_subject
    else:
        report_subject = "[正常]" + report_subject

    sender = task_detail.get("sender")

    if sender is None:
        log.warn("用户未配置发件人邮箱，测试报告将生成至本地")
    else:
        try:
            sent_mail(
                sender=sender,
                sender_alias=task_detail.get("senderAlias"),
                cc=task_detail.get("cc", []),
                to=task_detail.get("to"),
                subject=report_subject,
                body=report_content[0],
                att=report_content[1]
            )
        except:
            log.info(f"发送测试报告失败{traceback.format_exc()}")

    schedule_result["sender"] = sender
    schedule_result["sender_alias"] = task_detail.get("senderAlias", "pig_frame")
    schedule_result["to"] = task_detail.get("to")
    schedule_result["cc"] = task_detail.get("cc")
    schedule_result["subject"] = report_subject
    schedule_result["pass_case"] = sum_pass
    schedule_result["fail_case"] = sum_fail
    schedule_result["report_content"] = report_content
    schedule_result["begin_time"] = begin_time
    schedule_result["end_time"] = datetime.now()
    schedule_result["case_results"] = sum_results
    return schedule_result


def run_task_once(schedule_config="conf/scheduler.json", task_name=None, project_home=None):
    """
    读取任务配置启动定时任务
    @project_home 可自己指定项目根目录
    :return:
    """
    try:
        if project_home is None:
            project_home = get_project_home()
        task_json = f"{project_home}{os.sep}{schedule_config}"
        if not os.path.exists(task_json):
            print("5555")
            log.error(f"{schedule_config}文件不存在，请先配置任务执行脚本")

        with open(task_json, "r", encoding="utf-8") as file:
            task_list = json5.loads(file.read())
            for task in task_list:
                task["task_name"] = task_name
                cases = []
                for case in task.get("testcases"):
                    cases.append(
                        {
                            "case_file": os.path.abspath(
                                project_home
                                + f"{os.sep}testcases{os.sep}"
                                + case.get("file_path")
                            ),
                            "case_args": case.get("env_config_file"),
                            "kwargs": case.get("kwargs")
                            if case.get("kwargs") is not None
                            else {},
                        }
                    )

                run_cases(cases, task)
    except:
        log.error("解析task任务失败")
        log.error(traceback.format_exc())


def run_task(schedule_config="conf/scheduler.json"):
    """
    读取任务配置启动定时任务
    :return:
    """
    try:
        sys.path.append(os.getcwd())
        project_home = get_project_home()
        task_json = f"{project_home}{os.sep}{schedule_config}"
        if os.path.exists(task_json):
            with open(task_json, "r", encoding="utf-8") as file:
                task_obj = json5.loads(file.read())
            task_detail = task_obj
            t = Task()
            if task_detail is not None:
                for task in task_detail:
                    cases = []
                    for case in task.get("testcases"):
                        cases.append(
                            {
                                "case_file": os.path.abspath(
                                    project_home
                                    + f"{os.sep}testcases{os.sep}"
                                    + case.get("file_path")
                                ),
                                "case_args": case.get("env_config_file"),
                                "kwargs": case.get("kwargs")
                                if case.get("kwargs") is not None
                                else {},
                            }
                        )
                    t.add_task(
                        run_cases,
                        str(task.get("cron")).replace("?", "*"),
                        args=(cases, task),
                    )

                    if task.get("run_on_start"):
                        # 立即执行
                        run_cases(cases, task)
            log.info("定时任务已启动")
            t.start_tasks()
        else:
            log.info(f"{schedule_config}文件不存在，请先配置任务执行脚本")
    except:
        log.error("解析task任务失败")
        log.error(traceback.format_exc())


if __name__ == "__main__":
    run_task("templates/conf/scheduler.json")



# -*- coding: UTF-8 -*-
# @Time : 2023/9/5
# @Author : chengwenping2

from setuptools import setup
import pig_frame


def readme():
    with open("README.md", "r", encoding="utf-8") as f:
        return f.read()


setup(
    name="pig_sdk",
    version=pig_frame.__version__[0],
    description="接口自动化框架",
    long_description=readme(),
    keywords="自动化",
    packages=[
        "pig_frame",
        "pig_frame/log",
    ],
    package_data={
        "pig_frame": [
            "templates/*.*",
            "skeleton/*",
            "skeleton/*.*",
            "skeleton/**/*.*",
            "skeleton/**/**/*.*",
            "skeleton/**/**/**/*.*",
        ],
    },
    python_requires=">=3.9",
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Topic :: Utilities",
        # # 许可证信息
        # "License :: MIT License",
    ],
    url="https://git.oak.net.cn/test_group/pigs_sdk/-/tree/master/pig_frame",
    license="MIT",
    author="chengwenping2",
    author_email="chengwenping2@newhope.cn",
    maintainer="chengwenping2",
    maintainer_email="chengwenping2@newhope.cn",
    install_requires=[
        "jsonpath~=0.82",
        "PyMySQL~=1.0.3",
        "jmespath~=0.10.0",
        "Requests~=2.31.0",
        "setuptools~=56.0.0",
        "psutil~=5.9.5",
        "XMind~=1.2.0",
        "APScheduler~=3.10.1",
        "pyjson5~=1.6.2",
        "json5~=0.9.14",
        "Faker~=18.10.1",
        "Jinja2~=3.1.2",
        "termcolor~=2.3.0",
        "openpyxl~=3.1.2",
        "urllib3~=1.26.16",
        "beautifulsoup4~=4.12.2",
        "sshtunnel~=0.4.0",
        "redis~=5.0.0"
    ],
    extras_require={
      'GUI': ["PyQt6~=6.2.0"]
    },
    zip_safe=False,
)



# -*- coding: utf-8 -*-
# @Time : 2023/6/30
# @Author : chengwenping2
# @File : mail.py
"""
文件说明：使用smtplib接口发送邮件
"""

import traceback
from pig_frame import log
import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart


mail_host = "smtp.feishu.cn"  # 设置邮箱服务器
mail_pass = "UmaUOXgvvFLk7XLm"  # 这里就是邮箱的口令，也就是授权码，不是密码。

def sent_mail(sender, sender_alias, to, cc, subject, body, att):
    receivers = ",".join(to) + ",".join(cc)

    msg = MIMEMultipart("mixd")
    msg["Subject"] = subject
    msg["From"] = sender_alias
    msg["To"] = receivers

    alternative = MIMEMultipart('alternative')
    texthtml = MIMEText(body, _subtype='html', _charset='UTF-8')
    alternative.attach(texthtml)
    msg.attach(alternative)

    for i in att:
        HTMLpart = MIMEText(open(att[i],'rb').read(), 'html', 'UTF-8')
        HTMLpart['Content-Type'] = 'application/octet-stream'
        HTMLpart.add_header('Content-Disposition', 'attachment', filename=i+".html")
        msg.attach(HTMLpart)

    try:
        smtpObj = smtplib.SMTP_SSL(mail_host, 465)
        smtpObj.login(sender, mail_pass)
        smtpObj.sendmail(sender_alias, receivers.split(","), msg.as_string())
        smtpObj.quit()
        log.info(f"发送成功:{subject}")
        return True
    except smtplib.SMTPException as e:
        log.error(f"邮件发送失败{e}  \n{traceback.format_exc()}")
        return None

if __name__ == "__main__":
    sent_mail("chengwenping2@newhope.cn", "chengwenping2", ["chengwenping2@newhope.cn","18108278175@163.com"], [], "自动化巡检报告",
              "just a test",{"test1":r"D:\autotest\smartpig_api\test_report\01-测试物料\APP端登录并获取cookie值.html",
                             "test2":r"D:\autotest\smartpig_api\test_report\慧养猪功能巡检.html"})



# -*- coding: utf-8 -*-
# @Time : 2023/7/4
# @Author : chengwenping2

from jinja2 import Environment, PackageLoader
from pig_frame.num_util import calc_rate

jinja2_env = Environment(loader=PackageLoader('pig_frame', 'templates'))


def get_report_template(case_detail, case_pass, case_fail):
    template = jinja2_env.get_template('test_case_report.html')
    kwargs = {
        "echart_pass": case_pass,
        "echart_fail": case_fail,
        "alldataInstert": case_detail
    }
    return template.render(**kwargs)


def get_mail_html(total_scene, pass_case, fail_case, detail, fail_scene, use_time):
    for item in detail:
        item["rate"] = calc_rate(item["pass_case"], item["fail_case"])

    template = jinja2_env.get_template('test_suite_report.html')
    kwargs = {
        "total_scene": total_scene,
        "fail_scene": fail_scene,
        "use_time": use_time,
        "pass_case": pass_case,
        "fail_case": fail_case,
        "total_case": pass_case + fail_case,
        "rate": calc_rate(pass_case, fail_case),
        "cases": detail,
        "start_time": detail[0]["start_time"],
        "end_time": detail[-1]["end_time"],
    }
    return template.render(**kwargs)

def generate_json_template(**kwargs):
    template = jinja2_env.get_template('idl.json')
    return template.render(**kwargs)
